<?php 
class CRM_Cartable_CivirulesActions_BillingAddressVsHomeAddress extends CRM_Civirules_Action
{
    /**
     * Method to return the url for additional form processing for action
     * and return false if none is needed
     *
     * @param int $ruleActionId
     * @return bool
     * @access public
     */
    public function getExtraDataInputUrl($ruleActionId) {
        return FALSE;
    }

    /**
     * Method processAction to execute the action : billing address in home address if empty
     *
     * @param CRM_Civirules_TriggerData_TriggerData $triggerData
     * @access public
     *
     */
    public function processAction(CRM_Civirules_TriggerData_TriggerData $triggerData) { 
        $contactId = $triggerData->getContactId();
        if (empty($contactId)) {
            return;
        }

        $addressId = $triggerData->getEntityId();
        if (empty($addressId)) {
            return;
        }

        $result = civicrm_api4('Address', 'get', [
            'where' => [
                ['id', '=', $addressId],
            ],
            'limit' => 1,
            'checkPermissions' => FALSE,
        ]);

        if (empty($result) || $result->count() === 0) {
            return;
        }

        $addressData = $result[0];
        $addressBilling = $addressData['street_address'] ?? null;
        $cityBilling = $addressData['city'] ?? null;
        $postalCodeBilling = $addressData['postal_code'] ?? null;
        $countryIdBilling = $addressData['country_id'] ?? null;

        $locationTypeId = $addressData['location_type_id'] ?? null;
        $locationTypeLabelAPI = civicrm_api4('LocationType', 'get', [
            'select' => [
              'name',
            ],
            'where' => [
              ['id', '=', $locationTypeId],
            ],
            'limit' => 1,
            'checkPermissions' => FALSE,
          ]);
        $locationTypeLabel = $locationTypeLabelAPI[0]['name'] ?? null;

        $locationTypeHomeId = civicrm_api4('LocationType', 'get', [
            'select' => ['id'],
            'where' => [['name', '=', 'Home'],],
            'limit' => 1,
            'checkPermissions' => FALSE,
        ]);

        if ($locationTypeLabel == 'Billing') {
            $addressHome = civicrm_api4('Address', 'get', [
                'where' => [
                    ['contact_id', '=', $contactId],
                    ['location_type_id', '=', $locationTypeHomeId[0]['id']],
                ],
                'limit' => 1,
                'checkPermissions' => FALSE,
            ]);

            if (empty($addressHome[0]['id'])) {
                civicrm_api4('Address', 'create', [
                    'values' => [
                        'contact_id' => $contactId,
                        'location_type_id:label' => 'Domicile',
                        'street_address' => $addressBilling,
                        'postal_code' => $postalCodeBilling,
                        'city' => $cityBilling,
                        'country_id' => $countryIdBilling,
                    ],
                    'checkPermissions' => FALSE,
                ]);
            }
        }
    }

}