<?php 
class CRM_Cartable_CivirulesActions_MatchWorkAddressEmployer extends CRM_Civirules_Action
{
    /**
     * Method to return the url for additional form processing for action
     * and return false if none is needed
     *
     * @param int $ruleActionId
     * @return bool
     * @access public
     */
    public function getExtraDataInputUrl($ruleActionId) {
        return FALSE;
    }

    /**
     * Method processAction to execute the action : work address from employer address
     *
     * @param CRM_Civirules_TriggerData_TriggerData $triggerData
     * @access public
     *
     */
    public function processAction(CRM_Civirules_TriggerData_TriggerData $triggerData) { 
        $contactId = $triggerData->getContactId();
        $etablissement = "";
        $relationships = civicrm_api3('Relationship', 'get', [
            'return' => ["contact_id_b"],
            'contact_id_a' => $contactId,
            'relationship_type_id' => "Employee of",
            'is_active' => 1,
            'options' => ['limit' => 1, 'sort' => "id DESC"],
        ]);

        if ($relationships['count'] > 0) {
            $lastKey = key($relationships['values']);
            $contactIdB = $relationships['values'][$lastKey]['contact_id_b'];
            $contacts = civicrm_api3('Contact', 'get', [
                'return' => ["display_name"],
                'contact_type' => "Organization",
                'contact_sub_type' => "EducationalInstitution",
                'id' => $contactIdB,
                'options' => ['limit' => 1, 'sort' => "id DESC"],
              ]);
            if ($contacts['count'] > 0) {
                $lastKeyContact = key($contacts['values']);
                $etablissement = $contacts['values'][$lastKeyContact]['display_name'];
            } else {
                return false;
            }
        } else {
            return false;
        }

        $locationTypes = civicrm_api4('LocationType', 'get', [
            'checkPermissions' => false,
            'select' => ['id',],
            'where' => [['name', '=', 'travail'],],
        ]);


        $locationTypeWorkId = $locationTypes[0]['id'];

        $workAddressForContact = civicrm_api3('Address', 'get', [
            'checkPermissions' => false,
            'contact_id' => $contactId,
            'location_type_id' => $locationTypeWorkId,
          ]);

        $addresseEmployer = civicrm_api4('Address', 'get', [
            'select' => ['street_address','postal_code','city','country_id',],
            'where' => [['contact_id', '=', $contactIdB],],
            'checkPermissions' => false,
        ]);

        $street = $addresseEmployer[0]['street_address'];
        $cp = $addresseEmployer[0]['postal_code'];
        $city = $addresseEmployer[0]['city'];
        $country = $addresseEmployer[0]['country_id'];
        if (!empty($etablissement)) {
            $name_and_address = $etablissement . "\n" . $street;
            $address_data = [
                'checkPermissions' => false,
                'values' => [
                  'contact_id' => $contactId,
                  'location_type_id' => $locationTypeWorkId,
                  'street_address' => $name_and_address,
                  'city' => $city,
                  'postal_code' => $cp,
                  'country_id' => $country,
                ],
            ];
        } 
            

        if($workAddressForContact['count'] > 0) {
            //new employer => update
            $addressId = $workAddressForContact['id'];
            $address_data['values']['id'] = $addressId;
            civicrm_api4('Address', 'update', $address_data);
        }
        else {
            //no relation => create
            civicrm_api4('Address', 'create', $address_data);
        }
        
    }

}