<?php
class CRM_Cartable_CivirulesActions_SendEmailForPayments extends CRM_Civirules_Action
{
     /**
     * Method to return the url for additional form processing for action
     * and return false if none is needed
     *
     * @param int $ruleActionId
     * @return bool
     * @access public
     */
    public function getExtraDataInputUrl($ruleActionId) {
        return FALSE;
    }
    /**
     * Method processAction to execute the action : Send an email for sepa payments
     *
     * @param CRM_Civirules_TriggerData_TriggerData $triggerData
     * @access public
     *
     */
  public function processAction(CRM_Civirules_TriggerData_TriggerData $triggerData)
  {
    $op = $triggerData->getTrigger()->getOp();
    if($op !== 'create') {
      return;
    }
    $entityName = $triggerData->getEntity();
    $contributionId = isset($triggerData->getEntityData($entityName)['id']) ? $triggerData->getEntityData($entityName)['id'] : null;
    if($entityName === 'Contribution') {
      $contributions = civicrm_api4('Contribution', 'get', [
        'select' => [
          'contribution_recur_id',
          'contribution_page_id',
        ],
        'where' => [
          ['id', '=', $contributionId],
        ],
        'limit' => 1,
        'checkPermissions' => FALSE,
      ]);
      if (isset($contributions) && isset($contributions[0]['contribution_recur_id'])) {
        if($contributions[0]['contribution_recur_id'] !== null) {
          return;
        }
      }
    }
    $contributionPageId = isset($contributions) && isset($contributions[0]['contribution_page_id']) ? $contributions[0]['contribution_page_id'] : null;
    $contactId = $triggerData->getContactId() ? $triggerData->getContactId() : null;

    $contactEmails = civicrm_api4('Email', 'get', [
      'where' => [
        ['contact_id', '=', $contactId],
      ],
      'limit' => 1,
      'checkPermissions' => FALSE,
    ]);

    $contactMail = isset($contactEmails) && isset($contactEmails[0]['email']) ? $contactEmails[0]['email'] : '';

    if ($contactId !== null) {
      $relationships = civicrm_api4('Relationship', 'get', [
        'select' => [
          'contact_id_b',
          'contact_id_b.display_name',
        ],
        'where' => [
          ['contact_id_a', '=', $contactId],
          ['relationship_type_id:name', '=', 'SUD_Membre_De'],
        ],
        'limit' => 1,
        'checkPermissions' => FALSE,
      ]);
    }

    $syndicateId = isset($relationships) && isset($relationships[0]['contact_id_b']) ? $relationships[0]['contact_id_b'] : null;

    $receiptFromEmailAPI = civicrm_api4('ContributionPage', 'get', [
      'select' => [
        'receipt_from_email',
      ],
      'where' => [
        ['id', '=', $contributionPageId],
      ],
      'limit' => 1,
      'checkPermissions' => FALSE,
    ]);
    $receiptFromEmail = isset($receiptFromEmailAPI) && isset($receiptFromEmailAPI[0]['receipt_from_email']) ? $receiptFromEmailAPI[0]['receipt_from_email'] : 'notification-gestion@sudeducation.org';
    $syndicateMail = '';


    $locationTypes = civicrm_api4('LocationType', 'get', [
      'select' => [
        'id',
      ],
      'where' => [
        ['name', '=', 'Travail'],
      ],
      'limit' => 1,
      'checkPermissions' => FALSE,
    ]);

    $locationTypeId = isset($locationTypes) && isset($locationTypes[0]['id']) ? $locationTypes[0]['id'] : null;

    if ($syndicateId !== null && $locationTypeId !== null) {
      $syndicateEmails = civicrm_api4('Email', 'get', [
        'where' => [
          ['location_type_id', '=', $locationTypeId],
          ['contact_id', '=', $syndicateId],
        ],
        'checkPermissions' => FALSE,
      ]);
    }
    $messageTemplates = civicrm_api4('MessageTemplate', 'get', [
      'where' => [
        ['msg_title', '=', 'Contributions - Reçu de contribution contact'],
      ],
      'limit' => 1,
      'checkPermissions' => FALSE,
    ]);
    $template = isset($messageTemplates) && $messageTemplates[0] ? $messageTemplates[0] : null;
    $fromEmail = $receiptFromEmail;

    if ($template && $template['id']) {
      $params = [
        'tokenContext' => [
          'contactId' => $contactId,
          'contributionId' => $contributionId,
        ],
        'messageTemplateID' => $template['id'],
        'from' => $fromEmail,
        'toEmail' => $contactMail,
      ];
      
      $sent = CRM_Core_BAO_MessageTemplate::sendTemplate($params);
      // Civi::log()->debug("Cartable: Reçu de contribution contact. sent", $sent);
    }  

    if (isset($syndicateEmails) && isset($syndicateEmails[0]['email'])) {
      $messageTemplates = civicrm_api4('MessageTemplate', 'get', [
        'where' => [
          ['msg_title', '=', 'Contributions - Reçu de contribution syndicate'],
        ],
        'limit' => 1,
        'checkPermissions' => FALSE,
      ]);
      $template = isset($messageTemplates) ? $messageTemplates[0] : null;
      $syndicateMail = $syndicateEmails[0]['email'];
      if ($template && $template['id']) {
        $params = [
          'tokenContext' => [
            'contactId' => $contactId,
            'contributionId' => $contributionId,
          ],
          'messageTemplateID' => $template['id'],
          'from' => $fromEmail,
          'toEmail' => $syndicateMail,
        ];
        CRM_Core_BAO_MessageTemplate::sendTemplate($params);
      }
    }
  }

}