<?php
use CRM_Cartable_ExtensionUtil as E;

class CRM_Cartable_Page_GenerateForSyndicates extends CRM_Core_Page {

  public function run() {
    CRM_Utils_System::setTitle('Générer les données pour les syndicats');

    $nolimit = ['limit'=>0];
    $Syndicates = civicrm_api3('Contact', 'get', ['return'=>['id', 'display_name'], 'contact_sub_type'=>'SyndicatSUD', 'options'=>$nolimit]);
    $MembershipTypes = civicrm_api3('MembershipType', 'get', ['options'=>$nolimit]);
    $financialTypes = civicrm_api3('FinancialType', 'get', ['sequential' => 1,]);
    $campaigns = civicrm_api4('Contact', 'get', ['select' => ['id', 'campaign.title', 
    'DonneesSyndicat.CampaignPage',],'join' => [['Campaign AS campaign', 'LEFT', ['campaign.id', '=', 'DonneesSyndicat.CampaignPage']],],'where' => [['contact_sub_type', '=', 'SyndicatSUD'],],'limit' => 0,]);
    $managerGroup = civicrm_api4('Contact', 'get', ['select' => ['id', 'DonneesSyndicat.ManagerGroup', 'DonneesSyndicat.ManagerGroup:name',],'where' => [['contact_sub_type', '=', 'SyndicatSUD'],],]);
    $managerParentGroupId = civicrm_api4('Group', 'get', ['select' => ['id',],'where' => [['title', '=', 'Gestionnaires'],],]);
    $sepaCreditors = civicrm_api4('SepaCreditor', 'get', ['select' => ['id', 'name',],]);
    $paymentProcessorsIds = civicrm_api4('Contact', 'get', ['select' => ['id', 'display_name','DonneesSyndicat.PaymentProcessors',],'where' => [['contact_sub_type', '=', 'SyndicatSUD'],],]);
    $paymentProcessors = civicrm_api3('PaymentProcessor', 'get', ['sequential' => 1,]);
    $adhPages = civicrm_api4('Contact', 'get', ['select' => ['id', 'DonneesSyndicat.MembershipPage', 'contribution_page.title','contribution_page.id'],'join' => [['ContributionPage AS contribution_page', 'LEFT', ['contribution_page.id', '=', 'DonneesSyndicat.MembershipPage']],],'where' => [['contact_sub_type', '=', 'SyndicatSUD'],],]);
    $donPages = civicrm_api4('Contact', 'get', ['select' => ['id', 'DonneesSyndicat.DonationPage', 'contribution_page.title','contribution_page.id'],'join' => [['ContributionPage AS contribution_page', 'LEFT', ['contribution_page.id', '=', 'DonneesSyndicat.DonationPage']],],'where' => [['contact_sub_type', '=', 'SyndicatSUD'],],]);
    $paymentProcessorSystemPay = civicrm_api4('PaymentProcessor', 'get', ['select' => ['id',],'where' => [['title', '=', 'CB SystemPay National'],],]);
    $paymentProcessorNational = civicrm_api4('PaymentProcessor', 'get', ['select' => ['id',],'where' => [['title', '=', 'Prélèvement automatique (national)'],],]);
    $allManagerGroups = civicrm_api4('Group', 'get', ['select' => ['children',],'where' => [['title', '=', 'Gestionnaires'],],]);
    $relationshipManagerTypesID = civicrm_api4('RelationshipType', 'get', ['select' => ['id',],'where' => [['name_a_b', '=', 'SUD_Gestionnaire'],],]);
    $managersRelations = civicrm_api4('Relationship', 'get', ['select' => ['contact_id_a','contact_id_b',],'where' => [['relationship_type_id', '=', $relationshipManagerTypesID[0]['id']],],]);
    $allTeamGroups = civicrm_api4('Group', 'get', ['select' => ['id','children', 'title'],'where' => [['title', '=', 'Equipier⋅es de Sud éducation'],],]);
    $relationshipTeamTypesID = civicrm_api4('RelationshipType', 'get', ['select' => ['id',],'where' => [['name_a_b', '=', 'SUD_Equipiere_De'],],]);
    $teamsRelations = civicrm_api4('Relationship', 'get', ['select' => ['contact_id_a','contact_id_b',],'where' => [['relationship_type_id', '=', $relationshipTeamTypesID[0]['id']],],]);


    if ($allManagerGroups) {
      $managerGroupsArray = [];
      $childIds = [];
      $parentId = $allManagerGroups[0]['id'];
      foreach ($allManagerGroups as $childGroupManager) {
        $childIds = $childGroupManager['children'];
        if (!empty($childIds)) {
          foreach ($childIds as $childId) {
            $childGroupsResponse = civicrm_api4('Group', 'get', [
              'where' => [['id', '=', $childId]],
            ]);
            if (!empty($childGroupsResponse)) {

              $childTitle = $childGroupsResponse[0]['title'];
              $childId = $childGroupsResponse[0]['id'];
              $groupData = [
                'childId' => $childId,
                'childTitle' => $childTitle,
                'parentId' => $parentId,
              ];
              $managerGroupsArray[] = $groupData;
            }
          }
        }
      }
    }

    if ($allTeamGroups) {
      $teamGroupsArray = [];
      $childIds = [];
      $teamParentId = $allTeamGroups[0]['id'];
      $teamParentTitle = $allTeamGroups[0]['title'];
      foreach ($allTeamGroups as $childGroupTeam) {
        $childIds = $childGroupTeam['children'];
        if (!empty($childIds)) {
          foreach ($childIds as $childId) {
            $childGroupsResponse = civicrm_api4('Group', 'get', [
              'where' => [['id', '=', $childId]],
            ]);
            if (!empty($childGroupsResponse)) {

              $childTitle = $childGroupsResponse[0]['title'];
              $childId = $childGroupsResponse[0]['id'];
              $groupData = [
                'childId' => $childId,
                'childTitle' => $childTitle,
                'parentId' => $parentId,
              ];
              $teamGroupsArray[] = $groupData;
            }
          }
        }
      }
    }

    foreach($Syndicates['values'] as $id=>$Syndicate){
      //membershiptype
      $Syndicates['values'][$id]['membershipType'] = null;
      foreach($MembershipTypes['values'] as $membershipType){
          if($membershipType['member_of_contact_id'] == $Syndicate['id']){
              $Syndicates['values'][$id]['membershipType'] = $membershipType;
              break;
          }
      }
      //campaign
      $Syndicates['values'][$id]['campaignName'] = null;
      $Syndicates['values'][$id]['campaignId'] = null;
      foreach($campaigns as $campaign) {
        if(intval($campaign['id']) == intval($id)){
          if($campaign['DonneesSyndicat.CampaignPage'] !== null){
            $Syndicates['values'][$id]['campaignName'] = str_replace('_', ' ', $campaign['campaign.title']);
            $Syndicates['values'][$id]['campaignId'] = $campaign['DonneesSyndicat.CampaignPage'];
            break;
          }
        }
      }
      //Manager group
      $Syndicates['values'][$id]['managerGroupId'] = null;
      $Syndicates['values'][$id]['managerGroupName'] = null;
      foreach ($managerGroup as $manager) {
        if (intval($manager['id']) == intval($id)) {
          if ($manager['DonneesSyndicat.ManagerGroup'] !== null) {
            $Syndicates['values'][$id]['managerGroupId'] = $manager['DonneesSyndicat.ManagerGroup'];
            $Syndicates['values'][$id]['managerGroupName'] = $manager['DonneesSyndicat.ManagerGroup:name'];
            break;
          }
        }
      }
      //SepaCreditor
      $Syndicates['values'][$id]['sepaCreditorId'] = null;
      $Syndicates['values'][$id]['sepaCreditorName'] = null;
      foreach ($sepaCreditors as $sepaCreditor) {
        if($Syndicates['values'][$id]["display_name"] == $sepaCreditor['name']) {
          $Syndicates['values'][$id]['sepaCreditorId'] = $sepaCreditor['id'];
          $Syndicates['values'][$id]['sepaCreditorName'] = $sepaCreditor['name'];
        }
      }
      //Payment Procesor
      $Syndicates['values'][$id]['paymentProcessor'] = [];
      $processorOfSyndicate = false;
      if(count($paymentProcessorsIds) > 3) {
        $processorOfSyndicate = true;
      }
      foreach($paymentProcessorsIds as $paymentProcessorsId) {
        if($paymentProcessorsId['id'] == $id) {
        $processorIdsList = $paymentProcessorsId['DonneesSyndicat.PaymentProcessors'];
        $processorIdsArray = explode(",", $processorIdsList);
            foreach($processorIdsArray as $processorId) {
              if(!empty($processorId)) {
                foreach($paymentProcessors['values'] as $paymentProcessor) {
                  if($processorId == $paymentProcessor['id']) {
                    $Syndicates['values'][$id]['paymentProcessor'][$paymentProcessor['id']]= $paymentProcessor['name'];
                  }
                }
              }
              
            }
              
          }
        }
      //Adh Page
      $Syndicates['values'][$id]['adhPageId'] = null;
      $Syndicates['values'][$id]['adhPageName'] = null;
      foreach ($adhPages as $adhPage) {
        if (intval($adhPage['id']) == intval($id)) {
          if ($adhPage['contribution_page.id'] !== null) {
            $Syndicates['values'][$id]['adhPageId'] = $adhPage['contribution_page.id'];
            $Syndicates['values'][$id]['adhPageName'] = $adhPage['contribution_page.title'];
            break;
          }
        }
      }
       //Donation Page
       $Syndicates['values'][$id]['donPageId'] = null;
       $Syndicates['values'][$id]['donPageName'] = null;
       foreach ($donPages as $donPage) {
         if (intval($donPage['id']) == intval($id)) {
           if ($donPage['contribution_page.id'] !== null) {
             $Syndicates['values'][$id]['donPageId'] = $donPage['contribution_page.id'];
             $Syndicates['values'][$id]['donPageName'] = $donPage['contribution_page.title'];
             break;
           }
         }
       }

       //Managers
       $Syndicates['values'][$id]['managers'] = [];
       foreach($managersRelations as $managerRelation) {
          if(intval($managerRelation['contact_id_b']) == intval($id)) {
            $contact_id_a = $managerRelation['contact_id_a'];
            $contacts = civicrm_api4('Contact', 'get', ['select' => ['id','display_name'], 'where' => [['id', '=', $contact_id_a]],]);
            $managerData = [
                'id' => $contacts[0]['id'],
                'display_name' => $contacts[0]['display_name']
            ];
            $Syndicates['values'][$id]['managers'][] = $managerData;
          }
       }

       //Teams
        $Syndicates['values'][$id]['teams'] = [];
        foreach($teamsRelations as $teamRelation) {
           if(intval($teamRelation['contact_id_b']) == intval($id)) {
             $contact_id_a = $teamRelation['contact_id_a'];
             $contacts = civicrm_api4('Contact', 'get', ['select' => ['id','display_name'], 'where' => [['id', '=', $contact_id_a]],]);
             $teamData = [
                 'id' => $contacts[0]['id'],
                 'display_name' => $contacts[0]['display_name']
             ];
             $Syndicates['values'][$id]['teams'][] = $teamData;
           }
        }
    }
    $this->assign('syndicates', $Syndicates['values']);
    $this->assign('financialTypes', $financialTypes['values']);
    $this->assign('managerParentGroupId', $managerParentGroupId[0]['id']);
    $this->assign('processorOfSyndicate', $processorOfSyndicate);
    $this->assign('paymentProcessorSystemPayId', $paymentProcessorSystemPay[0]['id']);
    $this->assign('paymentProcessorNationalId', $paymentProcessorNational[0]['id']);
    $this->assign('teamParentGroupId', $teamParentId);
    $this->assign('teamParentTitle', $teamParentTitle);
    $this->assign('allManagerGroups', $managerGroupsArray);
    $this->assign('allTeamGroups', $teamGroupsArray);

    parent::run();
  }

}
