<?php

use Symfony\Component\Config\Resource\FileResource;
use Symfony\Component\DependencyInjection\ContainerBuilder;

require_once 'cartable.civix.php';
require_once __DIR__ . '/vendor/autoload.php';

// phpcs:disable
use CRM_Cartable_ExtensionUtil as E;
// phpcs:enable

define('CARTABLE_DIR', __DIR__);

WPReporting()->register('cartable', [
'label' => E::ts('Cartable'),
'description' => E::ts('Send error logs to developers.'),
'category' => 'plugin',
'to' => 'am91cm5hbGlzZXIrMjQyQGF2ZWNub3VzLmV1',
'only_in_dir' => __DIR__,
'javascript' => true,
]);
if(method_exists(WPReporting(), 'load_scripts')){
    WPReporting()->load_scripts();
    // Checkcing ja
    if(substr(WPReporting()->get_version(), 0, 3) < 1.5){
        add_action('admin_notices', function(){
            echo '<div class="notice notice-warning is-dismissible">
                <p>'.E::ts( 'Please update WP Reporting to the latest version. Please run:').'</p>
                <pre>composer update</pre>
            </div>';
        });
    }
}

require_once 'inc/utils.php';
/**
 * Implements hook_civicrm_config().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_config/
 */
function cartable_civicrm_config(&$config) {
  _cartable_civix_civicrm_config($config);
}

/**
 * Implements hook_civicrm_install().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_install
 */
function cartable_civicrm_install() {
  _cartable_civix_civicrm_install();
}

/**
 * Implements hook_civicrm_enable().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_enable
 */
function cartable_civicrm_enable() {
  _cartable_civix_civicrm_enable();
}

// --- Functions below this ship commented out. Uncomment as required. ---

/**
 * Implements hook_civicrm_preProcess().
 *
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_preProcess
 */
//function cartable_civicrm_preProcess($formName, &$form) {
//
//}

/**
 * Implements hook_civicrm_pageRun().
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_pageRun/
 */
function cartable_civicrm_pageRun(  &$page ) {
    \WPReporting()->listen('cartable');
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/cartable.js', 10, 'html-header', false);
    if(is_admin()){
        CRM_Core_Resources::singleton()->addStyleFile('cartable', 'assets/built/admin.css');
    }
    $currentUrl = CRM_Utils_System::currentPath();
    if ($currentUrl === 'civicrm/sud/help/documentation' || $currentUrl === 'civicrm/sud/help/faq') {
        CRM_Core_Resources::singleton()->addStyleFile('cartable', 'assets/built/help.css');
    }
    if($page instanceof CRM_Cartable_Page_GenerateForSyndicates){
        CRM_Core_Resources::singleton()->addScriptFile('cartable', 'vendor/agencenous/jquery-tabs/jquery-tabs.js', 100, 'html-header', false);
        CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/generate-for-syndicates.js', 110, 'html-header', false);
        CRM_Core_Resources::singleton()->addStyleFile('cartable', 'vendor/agencenous/jquery-tabs/jquery-tabs.css');
    }
    if($page instanceof CRM_Cartable_Page_SelfManagementSettings){
        CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/self-management-settings.js', 110, 'html-header', false);
    }
    if($page instanceof CRM_Contact_Page_View_Summary
    || $page instanceof CRM_Contact_Page_View_Activities
    || $page instanceof CRM_Contact_Page_View_Relationships
    || $page instanceof CRM_Contact_Page_View_CustomData
    || $page instanceof CRM_Member_Page_Tab) {
        CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/contribution-counter.js');
    }
    if($page instanceof CRM_Contribute_Form_Contribution_Main){
        $page->assiign('is_recur', true);
    }
    if($page instanceof CRM_Contact_Page_View_UserDashBoard){
        $contribute_rows =  $page->get_template_vars('contribute_rows');
        // CiviCRM uses the default contributonPage
        // We need to set the contribution page ID of the correct syndicate
        foreach($contribute_rows as $k => $row){
            if(isset($row['buttons']) && isset($row['buttons']['pay'])){
                // Check if a membership is associated to this contribution
                if(isset($row['contribution_recur_id']) && $row['contribution_recur_id']){
                    $maybe_membership = \Civi\Api4\Membership::get(false)
                        ->addSelect('membership_type_id.member_of_contact_id')
                        ->addWhere('contribution_recur_id', '=', $row['contribution_recur_id'])
                        ->setLimit(1)
                        ->execute()
                        ->first();
                        
                    if ($maybe_membership && isset($maybe_membership['membership_type_id.member_of_contact_id']) && $maybe_membership['membership_type_id.member_of_contact_id']) {
                        $maybe_syndicate_pages = \Civi\Api4\Contact::get(false)
                        ->addSelect('DonneesSyndicat.MembershipPage')
                        ->addSelect('DonneesSyndicat.DonationPage')
                        ->addWhere('id', '=', $maybe_membership['membership_type_id.member_of_contact_id'])
                        ->setLimit(1)
                        ->execute()
                        ->first();
                        $contribute_rows[$k]['buttons']['pay']['url'] = CRM_Utils_System::url('civicrm/contribute/transact', [
                            'reset' => 1,
                            'id' => $maybe_syndicate_pages['DonneesSyndicat.MembershipPage'],
                            'ccid' => $row['id'],
                            // 'cs' => $page->getUserChecksum(),
                            'cid' => $row['contact_id'],
                        ]);
                    }

                }
            }
        }
        $page->assign('contribute_rows', $contribute_rows);
    }
    if($page instanceof CRM_Sepa_Page_MandateTab){
        $rcur_list =  $page->get_template_vars('rcurs');
        foreach($rcur_list as $rcur_id=>$rcur){
            $sepa_mandates = civicrm_api4('SepaMandate', 'get', [
                'where' => [
                    ['id', '=', $rcur['mandate_id']],
                ],
                'limit' => 1,
                'checkPermissions' => false,
            ]);
            if(isset($sepa_mandates[0]) && ($sepa_mandate = $sepa_mandates[0]) && $sepa_mandate['entity_table'] === 'civicrm_contribution_recur'){
                $contribution_recur_id = $sepa_mandate['entity_id'];
                $memberships = civicrm_api4('Membership', 'get', [
                    'select' => [
                        'membership_type_id:label',
                        'Parametres.numberOfMonthsForContribution',
                        'Parametres.numberOfMonthsForProrata',
                        'Parametres.GrilleSalariale',
                        'Parametres.PaymentMethod',
                    ],
                    'where' => [
                        ['contribution_recur_id', '=', $contribution_recur_id],
                    ],
                    'limit' => 1,
                    'checkPermissions' => false,
                ]);
                if($memberships && isset($memberships[0])){
                    $membership = $memberships[0];
                    if(isset($membership['Parametres.numberOfMonthsForProrata']) && is_numeric($membership['Parametres.numberOfMonthsForProrata'])){
                        $amount = (float) $rcur['amount'];
                        $new_total_amount = $amount * $membership['Parametres.numberOfMonthsForProrata'];
                        $rcur_list[$rcur_id]['total_amount'] = $new_total_amount;
                        if(strstr($membership['Parametres.PaymentMethod'], '_apportionment')){
                            $rcur_list[$rcur_id]['frequency'] .= ' '.sprintf(E::ts('(prorata %d months)') ,$membership['Parametres.numberOfMonthsForProrata']);
                        }
                    }
                }
            }

        }
        $page->assign('rcurs', $rcur_list);
    }
    $MembershipToComplete_id = cartable_get_membership_to_complete_id();
    if($MembershipToComplete_id){
        // Maybe do something here
    }
    \WPReporting()->stop();
}

/**
 * Implements hook_civicrm_alterRedirect().
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_alterRedirect/
 */
function cartable_civicrm_alterRedirect(&$url, &$context){
    $query = $url->getQuery();

    $MembershipToComplete_id = cartable_get_membership_to_complete_id();
    if($MembershipToComplete_id){
        $query.="&membership-to-complete={$MembershipToComplete_id}";
    }
    $MembershipFirstAmount = cartable_get_membership_first_amount();
    if($MembershipFirstAmount){
        $query.="&membership-first-amount={$MembershipFirstAmount}";
    }

    $url = $url->withQuery($query);
}

/**
 * Implements hook_civicrm_navigationMenu().
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_navigationMenu
 */
function cartable_civicrm_navigationMenu(&$menu) {
 _cartable_civix_insert_navigation_menu($menu, 'Support', [
    'label' => 'Aide SUD éducation',
    'name' => 'SUD Aide',
    'url' => 'civicrm/sud/help/documentation',
    'permission' => 'access CiviCRM',
    'operator' => 'OR',
    'separator' => 0,
  ]);
  _cartable_civix_insert_navigation_menu($menu, 'Support', [
    'label' => 'FAQ SUD éducation',
    'name' => 'SUD F.A.Q',
    'url' => 'civicrm/sud/help/faq',
    'permission' => 'access CiviCRM',
    'operator' => 'OR',
    'separator' => 0,
  ]);
 _cartable_civix_insert_navigation_menu($menu, 'Support', [
   'label' => 'Champs SUD éducation',
   'name' => 'SUD Aide',
   'url' => 'civicrm/sud/help/fields',
   'permission' => 'access CiviCRM',
   'operator' => 'OR',
   'separator' => 0,
 ]);
 _cartable_civix_navigationMenu($menu);
}

/**
 * Implements alter_next_collection_date().
 * This hook is called by the alternativeBatching:
 *  you can set a custom collection date for a rcurring contribution.
 *  For example you can use this hook when you mandate is connected to a yearly membership from January to December.
 *  And when a new member signs up in October. You want to collect that money in october and the membership will end on 31st of December.
 *  So the next collection is in January.
 *
 * @param string $next_collection_date  the calculated collection date (format: "YYYY-MM-DD").
 * @param array  $data array with data (such as mandate_id, mandate_entity_id for contribution recur id).
 * @link https://github.com/Project60/org.project60.sepa/#customisation
 */
function cartable_civicrm_alter_next_collection_date(&$return_date, $rcontribution){
    \WPReporting()->listen('cartable');
    $contribution_recur_id = isset($rcontribution['mandate_entity_id']) ? $rcontribution['mandate_entity_id'] : null;
    $creditor_id = isset($rcontribution['mandate_creditor_id']) ? $rcontribution['mandate_creditor_id'] : null;
    if(!$contribution_recur_id || !$creditor_id){
        return;
    }
    try{
        $sepaCreditor = \Civi\Api4\SepaCreditor::get(false)
        ->addWhere('id', '=', $creditor_id)
        ->setLimit(1)
        ->execute();
    }
    catch(\Throwable $e){
        \WPReporting()->send($e, 'cartable');
        return;
    }
    catch(\Exception $e){
        \WPReporting()->send($e, 'cartable');
        return;
    }

    $sepaCreditor = $sepaCreditor->first();

    try{
        $syndicate = $contacts = \Civi\Api4\Contact::get(false)
        ->addSelect(
            'ParametresAdhesion.ContributionStartDay', 
            'ParametresAdhesion.ContributionStartMonth',
            'ParametresAdhesion.ContributionEndDay',
            'ParametresAdhesion.ContributionEndMonth'
        )
        ->addWhere('id', '=', $sepaCreditor['creditor_id'])
        ->setLimit(1)
        ->execute();
    }
    catch(\Throwable $e){
        \WPReporting()->send($e, 'cartable');
        return;
    }
    catch(\Exception $e){
        \WPReporting()->send($e, 'cartable');
        return;
    }

    $syndicate = $syndicate->first();

    // Check if this syndicates requires a custom collection date
    $required_fields = [
        'ParametresAdhesion.ContributionStartDay', 
        'ParametresAdhesion.ContributionStartMonth',
        'ParametresAdhesion.ContributionEndDay',
        'ParametresAdhesion.ContributionEndMonth',
    ];
    foreach($required_fields as $field){
        if(!isset($syndicate[$field]) || empty($syndicate[$field])){
            return;
        }
    }

    $return_date = cartable_compute_next_contribution_date(
        $return_date, 
        $syndicate['ParametresAdhesion.ContributionStartMonth'],
        $syndicate['ParametresAdhesion.ContributionStartDay'],
        $syndicate['ParametresAdhesion.ContributionEndMonth'],
        $syndicate['ParametresAdhesion.ContributionEndDay']
    );

    \WPReporting()->stop();
}

/**
 * Implements hook_civicrm_apiWrapper().
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_apiWrapper/
 */
function cartable_civicrm_apiWrappers(&$wrappers, $apiRequest) {
    if ($apiRequest['entity'] == 'SepaTransactionGroup' && $apiRequest['action'] == 'getdetail') {
        if ($apiRequest['version'] == '3') {
            $wrappers[] = new CRM_Cartable_API3Wrappers_SepaTransactionGroup();
          }
          elseif ($apiRequest['version'] == '4') {
            $wrappers[] = new CRM_Cartable_API4Wrappers_SepaTransactionGroup();
          }
    }
}

/**
 * Add recurring contributions if needed.
 * @see https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_pre/
 */
function cartable_civicrm_pre($op, $objectName, $id, &$params){
    \WPReporting()->listen('cartable');
    if (!in_array($objectName, [
            'Contribution', 
            'ContributionRecur', 
            'LineItem', 
            'Membership', 
            'Individual',
        ]) || 
        !in_array($op, [
            'create', 
            'edit',
        ])) {
        return;
    }
    
    if($objectName == 'Individual'){
        $account_holder = isset($params['account_holder']) ? $params['account_holder'] : null;
        $bank_account_number = isset($params['bank_account_number']) ? $params['bank_account_number'] : null;
        $bank_identification_number = isset($params['bank_identification_number']) ? $params['bank_identification_number'] : null;
        $bank_name = isset($params['bank_name']) ? $params['bank_name'] : null;
        if($account_holder && $bank_account_number && $bank_identification_number && $bank_name){
            $contact = civicrm_api4('Contact', 'update', [
                'values' => [
                    'ParametresPaiement.SEPA_Holder' => $account_holder,
                    'ParametresPaiement.SEPA_IBAN' => $bank_account_number,
                    'ParametresPaiement.SEPA_BIC' => $bank_identification_number,
                    'ParametresPaiement.SEPA_Bank' => $bank_name,
                ],
                'where' => [
                    ['id', '=', $params['contact_id']],
                ],
                'checkPermissions' => false,
            ]);
        }
        //delete contact work address if employer is deleted
        if($op === 'edit') {
            $contactId = $params['contact_id'];
            $relationships = civicrm_api3('Relationship', 'get', [
                'return' => ["contact_id_b", 'is_active'],
                'contact_id_a' => $contactId,
                'relationship_type_id' => "Employee of",
                'options' => ['limit' => 1, 'sort' => "id DESC"],
            ]);
            if ($relationships['count'] > 0) {
                $lastKey = key($relationships['values']);
                $relationStatus = $relationships['values'][$lastKey]['is_active'];

                $locationTypes = civicrm_api4('LocationType', 'get', [
                    'select' => ['id',],
                    'where' => [['name', '=', 'travail'],],
                    'checkPermissions' => false,
                ]);
                $locationTypeWorkId = $locationTypes[0]['id'];

                //relation exist but not active => delete work address
                if(!$relationStatus) {
                    civicrm_api4('Address', 'delete', [
                        'where' => [
                            ['contact_id', '=', $contactId],
                            ['location_type_id', '=', $locationTypeWorkId],
                        ],
                        'checkPermissions' => false,
                    ]);
                }
            }
        }
    }

    if($objectName == 'Membership'){
        $MembershipToComplete_id = cartable_get_membership_to_complete_id();
        if($op == 'create'){
            if($MembershipToComplete_id){
                // Here we want to update the existing membership instead of create a new one
                $params['id'] = $MembershipToComplete_id;
                // We also set the "old" contribution_retur status to complete
                $MembershipToComplete = cartable_get_membership_to_complete($MembershipToComplete_id);
                if($MembershipToComplete){
                    civicrm_api4('ContributionRecur', 'update', [
                        'values' => [
                            'contribution_status_id:name' => 'Completed',
                        ],
                        'where' => [
                            ['id', '=', $MembershipToComplete['contribution_recur_id']],
                        ],
                        'checkPermissions' => false,
                    ]);
                }
            }
        }
        if($op == 'edit'){
            // Membership is edited
            // We want to ensure the end date  is still consistent for multiple payments on a single membership
            // error_log('DEBUG: pre '.$op.' '.$objectName.': '.json_encode($params, true));

            // Get the membership contribution_recur_id
            $membership = civicrm_api4('Membership', 'get', [
                'select' => [
                    'id',
                    'contribution_recur_id',
                    'membership_type_id',
                    'start_date',
                    'end_date',
                ],
                'where' => [
                    ['id', '=', $id],
                ],
                'limit' => 1,
                'checkPermissions' => false,
            ]);
            if(!isset($membership[0])){
                return;
            }
            $membership = array_merge($membership[0], $params);
            // In some cases, start_date, end_date and join_date are formatted without dash. Lets convert yyyymmdd into yyyy-mm-dd
            if(isset($membership['start_date']) && is_numeric($membership['start_date']) && strlen($membership['start_date']) == 8){
                $membership['start_date'] = substr($membership['start_date'], 0, 4).'-'.substr($membership['start_date'], 4, 2).'-'.substr($membership['start_date'], 6, 2);
            }
            if(isset($membership['end_date']) && is_numeric($membership['end_date']) && strlen($membership['end_date']) == 8){
                $membership['end_date'] = substr($membership['end_date'], 0, 4).'-'.substr($membership['end_date'], 4, 2).'-'.substr($membership['end_date'], 6, 2);
            }
            if(isset($membership['join_date']) && is_numeric($membership['join_date']) && strlen($membership['join_date']) == 8){
                $membership['join_date'] = substr($membership['join_date'], 0, 4).'-'.substr($membership['join_date'], 4, 2).'-'.substr($membership['join_date'], 6, 2);
            }
            
            cartable_log('Merged Membership', $membership);
            if(isset($membership['contribution_recur_id']) && $membership['contribution_recur_id']){
                // Get the membership enddate
                $endate = strtotime($membership['end_date']);

                // Get the recurring contribution associated to this membership
                $contribution_recur_id = $membership['contribution_recur_id'];
                $contribution_recur = civicrm_api4('ContributionRecur', 'get', [
                    'select' => [
                        'id',
                        'start_date',
                        'end_date',
                        'frequency_unit',
                        'frequency_interval',
                        'installments',
                    ],
                    'where' => [
                        ['id', '=', $contribution_recur_id],
                    ],
                    'limit' => 1,
                    'checkPermissions' => false,
                ]);
                if(!isset($contribution_recur[0])){
                    return;
                }
                $contribution_recur = $contribution_recur[0];

                // If the frequency unit is year, do nothing.
                // The end date is already consistent with the membership end date
                if($contribution_recur['frequency_unit'] == 'year'){
                    cartable_log('Yearly contribution. No need to alter membership date.', $contribution_recur);
                    return;
                }
                cartable_log('Non yearly contribution. Maybe need to alter membership date.', $contribution_recur);

                // Get the membership type
                $membership_type_id = $membership['membership_type_id'];
                $membership_type = civicrm_api4('MembershipType', 'get', [
                    'select' => [
                        'id',
                        'duration_unit',
                        'duration_interval',
                        'period_type',
                        'fixed_period_start_day',
                        'fixed_period_rollover_day',
                    ],
                    'where' => [
                        ['id', '=', $membership_type_id],
                    ],
                    'limit' => 1,
                    'checkPermissions' => false,
                ]);
                cartable_log('Membership type', $membership_type);
                $membership_type = $membership_type[0];
                
                $expected_end_date = null;

                // Compute the expected end date dependng on the membership type
                // 1. if the membership type has a fixed period
                // We compute the end date based on the membership date
                // Expected enddate is: (fixedPeriod.day, fixedPeriod.month, (membership.startDate.year + 1 year)) - 1 day
                if($membership_type['period_type'] == 'fixed'){
                    $expected_day = substr($membership_type['fixed_period_start_day'], -2);
                    $expected_month = substr($membership_type['fixed_period_start_day'], -3, -2);
                    $expected_year = date('Y', strtotime($membership['start_date'])) + 1;
                    cartable_log('Expected stuff', [
                        'expected_day' => $expected_day,
                        'expected_month' => $expected_month,
                        'expected_year' => $expected_year,
                    
                    ]);
                    $expected_end_date = date('Y-m-d', strtotime('-1 day', mktime(0, 0, 0, $expected_month, $expected_day, $expected_year)));
                }
                // 2. if the membership type has a rolling period
                // We compute the end date based on the membership start date
                // Expected enddate is: (membership.startDate + 1 year) - 1 day
                elseif($membership_type['period_type'] == 'rolling'){
                    $expected_end_date = date('Y-m-d', strtotime(' +1 year -1 day', strtotime($membership['start_date'])));
                }

                cartable_log('Expected end date', $expected_end_date);
                if($expected_end_date){
                    $params['end_date'] = $expected_end_date;
                }

                // Check if the contribution date is an anniversary of the membership
                // If it is, we change the start_date to apply the start_datte depending on the membership_type
                // if the contribution.date is after the membership.end_date
                // - rolling case: membership.start_date + 1 year
                // - fixed case: fixedPeriod.day, fixedPeriod.month, (membership.startDate.year + 1 year)
                $expected_start_date = null;

                // Get the latest contribition associated to this recurring contribution
                $contribution = civicrm_api4('Contribution', 'get', [
                    'select' => [
                        'id',
                        'receive_date',
                    ],
                    'where' => [
                        ['contribution_recur_id', '=', $contribution_recur_id],
                    ],
                    'orderBy' => [
                        'receive_date' => 'DESC',
                    ],
                    'limit' => 1,
                    'checkPermissions' => false,
                ]);
                if(!isset($contribution[0])){
                    return;
                }
                $contribution = $contribution[0];
                cartable_log('Contribution', $contribution);

                $contribution_date = strtotime($contribution['receive_date']);
                $membership_start_date = strtotime($membership['start_date']);
                $membership_end_date = strtotime($membership['end_date']);
                
                cartable_log('Check for membership renewal requirement', [
                    'contribution_date' => $contribution['receive_date'],
                    'membership_start_date' => $membership['start_date'],
                    'membership_end_date' => $membership['end_date'],
                ]);
                // Compare dates
                if($contribution_date >= $membership_end_date){
                    // 1. if the membership type has a fixed period
                    if($membership_type['period_type'] == 'fixed'){
                        $expected_day = substr($membership_type['fixed_period_start_day'], -2);
                        $expected_month = substr($membership_type['fixed_period_start_day'], -3, -2);
                        $expected_year = date('Y', strtotime($membership['start_date'])) + 1;
                        cartable_log('Expected stuff', [
                            'expected_day' => $expected_day,
                            'expected_month' => $expected_month,
                            'expected_year' => $expected_year,
                        
                        ]);
                        $expected_start_date = mktime(0, 0, 0, $expected_month, $expected_day, $expected_year);
                    }
                    // 2. if the membership type has a rolling period
                    elseif($membership_type['period_type'] == 'rolling'){
                        $expected_start_date = strtotime(' +1 year', strtotime($membership['start_date']));
                    }
                    $expected_end_date = strtotime(' +1 year -1 day', $expected_start_date);
                    $params['start_date'] = date('Y-m-d', $expected_start_date);
                    $params['end_date'] = date('Y-m-d', $expected_end_date);
                }
            }
            cartable_log('Final membership params', $params);
        }
    }

    if ($objectName == 'ContributionRecur') {
        $PaymentMethodField = cartable_get_custom_field_id('PaymentMethod', 'Parametres');
        $numberOfMonthsForProrataField = cartable_get_custom_field_id('numberOfMonthsForProrata', 'Parametres');
        if(isset($params['frequency_unit']) && isset($_POST[$PaymentMethodField])){
            $frequency = $_POST[$PaymentMethodField];
            $frequency = str_replace('ly', '', $frequency);
            $params['frequency_unit'] = str_replace('_apportionment', '', $frequency);
        }
        $GrilleSalarialeField = cartable_get_custom_field_id('GrilleSalariale', 'Parametres');
        if(isset($params['amount']) && isset($_POST[$GrilleSalarialeField]) && strstr($_POST[$GrilleSalarialeField], '|')){ // $params['amount'] == 0 && 
            $raw_grid = explode('|', $_POST[$GrilleSalarialeField]);
            $params['amount'] = $raw_grid[1];
            if($params['frequency_unit'] == 'month'){
                $numberOfMonthsToDivide = (false != $nb = filter_input(INPUT_POST, $numberOfMonthsForProrataField, FILTER_SANITIZE_NUMBER_INT)) ? $nb : 12;
                $params['amount'] = $params['amount']/$numberOfMonthsToDivide;
                if($nb){
                   // TODO: Check if this is a good idea for a "no end date" contribution
                   // $params['installments'] = $nb;
                }
            }
        }
    }
    if ($objectName == 'Contribution' && $op == 'create') {
        $MembershipToComplete_id = cartable_get_membership_to_complete_id();
        $MembershipFirstAmount = cartable_get_membership_first_amount();
        if($MembershipToComplete_id && $MembershipFirstAmount){
            // $MembershipToComplete = cartable_get_membership_to_complete($MembershipToComplete_id);
            $params['total_amount'] = $MembershipFirstAmount;
            $params['net_amount'] = $MembershipFirstAmount;
        }
    }
    if ($objectName == 'LineItem' && $op == 'create') {
        $MembershipToComplete_id = cartable_get_membership_to_complete_id();
        $MembershipFirstAmount = cartable_get_membership_first_amount();
        if($MembershipToComplete_id && $MembershipFirstAmount){
            $params['qty'] = $MembershipFirstAmount / (float) $params['line_total'];
            $params['line_total'] = $MembershipFirstAmount;
        }
    }
    \WPReporting()->stop();
}

/**
 * use civicrm_post to add member of relationship on contribution
 * @param mixed $op
 * @param mixed $objectName
 * @param mixed $objectId
 * @param mixed $objectRef
 * @return void
 */
function cartable_civicrm_post($op, $objectName, $objectId, &$objectRef) {
    \WPReporting()->listen('cartable');
    if ($op == 'create' && $objectName == 'Contribution') {
        //member of
        $relationshipTypes = civicrm_api4('RelationshipType', 'get', [
            'select' => [
                'id',
            ],
            'where' => [
                ['name_a_b', '=', 'SUD_Membre_De'],
            ],
            'checkPermissions' => FALSE,
        ]);
        $relationTypeId = $relationshipTypes[0]['id'];

        //contact
        $contactId = $objectRef->contact_id;

        // Source
        $source = $objectRef->source;

        //syndicate
        $membershipTypeValues = null;
        
        if(isset($_POST['membership_type_id'][0]) && is_numeric($_POST['membership_type_id'][0])){
            $membershipTypeValues = $_POST['membership_type_id'][0];
        }
        
        $contributionPageId = $objectRef->contribution_page_id;
        if($contributionPageId){
            $syndicate = null;

            if (strpos($source, 'Adh') !== false) {
                $syndicate = civicrm_api4('Contact', 'get', [
                    'select' => [
                      'id',
                    ],
                    'where' => [
                      ['contact_sub_type', '=', 'SyndicatSUD'],
                      ['DonneesSyndicat.MembershipPage', '=', $contributionPageId],
                    ],
                    'checkPermissions' => false,
                ]);
            } else if(strpos($source, 'Don') !== false) {
                $syndicate = civicrm_api4('Contact', 'get', [
                    'select' => [
                      'id',
                    ],
                    'where' => [
                      ['contact_sub_type', '=', 'SyndicatSUD'],
                      ['DonneesSyndicat.DonationPage', '=', $contributionPageId],
                    ],
                    'checkPermissions' => false,
                ]);
            }
            if($syndicate && $syndicate->rowCount > 0) {
                $membershipTypeValues =  $syndicate[0]['id'];
            }
        }

        if($membershipTypeValues){

            $relationships = civicrm_api4('Relationship', 'get', [
                'where' => [
                  ['relationship_type_id', '=', $relationTypeId],
                  ['contact_id_a', '=', $contactId],
                  ['contact_id_b', '=', $membershipTypeValues],
                ],
                'checkPermissions' => false,
            ]);
            if($relationships->rowCount == 0) {
                //create relationship
                $relationshipParams = array(
                    'relationship_type_id' => $relationTypeId,
                    'contact_id_a' => $contactId,
                    'contact_id_b' => $membershipTypeValues,
                );
                civicrm_api4('Relationship', 'create', [
                    'values' => $relationshipParams,
                    'checkPermissions' => false
                ]);
            }
        }
       
    }

    if ($op == 'edit' && $objectName == 'Contribution') {
        // Expected contribution statuses
        $allowed_contribution_statuses_name = [
            'Completed',
        ];
        // get value of allowed contribution statuses
        $allowed_contribution_statuses = civicrm_api4('OptionValue', 'get', [
            'select' => [
                'value',
            ],
            'where' => [
                ['option_group_id:name', '=', 'contribution_status'],
                ['name', 'IN', $allowed_contribution_statuses_name],
            ],
            'checkPermissions' => false,
        ]);
        $allowed_contribution_statuses = $allowed_contribution_statuses->column('value');

        // Contribution has an elligible status
        if($objectRef->contribution_status_id && in_array($objectRef->contribution_status_id, $allowed_contribution_statuses)){
            $where = null;

            // Get membership directly fron contribution
            $query = "SELECT membership_id FROM civicrm_membership_payment WHERE contribution_id={$objectId}";
            $civicrm_membership_payment_dao = CRM_Core_DAO::executeQuery($query);
            $membership_id = $civicrm_membership_payment_dao->fetchAll();
            if($membership_id && isset($membership_id[0]) && isset($membership_id[0]['membership_id'])){
                $membership_id = $membership_id[0]['membership_id'];
                $where = [
                    ['id', '=', $membership_id],
                ];
            }
            // Fallback on memebership via recurring contribution
            elseif(isset($objectRef->contribution_recur_id) && $objectRef->contribution_recur_id){
                $where = [
                    ['contribution_recur_id', '=', $objectRef->contribution_recur_id],
                ];
            }

            // Continue if we have criteria to find a membership
            if($where){
                try{
                    $maybe_membership = civicrm_api4('Membership', 'get', [
                        'select' => [
                            'id',
                            'membership_type_id',
                            'membership_status.is_current_member',
                            'membership_status.name',
                            'start_date',
                            'end_date',
                            'join_date',
                        ],
                        'join' => [
                            ['MembershipStatus AS membership_status', 'LEFT', ['status_id', '=', 'membership_status.id']],
                        ],
                        'where' => $where,
                        'limit' => 1,
                        'checkPermissions' => false,
                    ]);
                }
                catch(\Throwable $e){
                    \WPReporting()->send($e, 'cartable');
                }
                catch(\Exception $e){
                    \WPReporting()->send($e, 'cartable');
                }
                // A membership is found
                if($maybe_membership && isset($maybe_membership[0])){
                    $membership = $maybe_membership[0];
                    $need_update = [];

                    $expected_join_date = $membership['join_date'];
                    $expected_start_date = $membership['start_date'];
                    $expected_end_date = $membership['end_date'];

                    // Update join & start dates if empty
                    if(empty($membership['join_date'])){
                        $expected_join_date = $need_update['join_date'] = $objectRef->receive_date;
                    }
                    if(empty($membership['start_date'])){
                        $expected_start_date = $need_update['start_date'] = $objectRef->receive_date;
                    }

                    // Ensure start_date if after join_date
                    if(strtotime($expected_join_date) > strtotime($expected_start_date)){
                        $need_update['start_date'] = $expected_join_date;
                    }

                    // Renewal for passed contributions make erroneous end dates
                    // $calcDates = CRM_Member_BAO_MembershipType::getRenewalDatesForMembershipType(
                    //     $membership['id'],
                    //     $objectRef->receive_date,
                    //     $membership['membership_type_id'],
                    //     1,
                    // );

                    // Get new end date
                    // WARNING: This may cause issues on rolling membership_types
                    $newDates = CRM_Member_BAO_MembershipType::getDatesForMembershipType(
                        $membership['membership_type_id'],
                        $objectRef->receive_date,
                    );
                    if($newDates && isset($newDates['end_date']) && strtotime($newDates['end_date']) > strtotime($membership['end_date'])){
                        $expected_end_date = $need_update['end_date'] = $newDates['end_date'];
                    }

                    // Ensure end_date if after start_date
                    if(strtotime($expected_start_date) > strtotime($expected_end_date)){
                        $need_update['end_date'] = '';
                    }

                    // Update status even if dates are not modified
                    if($membership['membership_status.is_current_member'] == false){
                        $need_update['status_id'] = '';
                    }

                    // Update memebership if needed
                    if(!empty($need_update)){
                        // We want to force the status computation. So let's empty it
                        $need_update['status_id'] = '';
                        $update = civicrm_api4('Membership', 'update', [
                            'values' => $need_update,
                            'where' => [
                                ['id', '=', $membership['id']],
                            ],
                            'checkPermissions' => false,
                        ]);
                    }
                }

            }
        }
    }
    \WPReporting()->stop();
}

/**
 * Implements hook_civicrm_alterPaymentProcessorParams().
 * @see OmnipayPaymentProcessor
 */
function cartable_civicrm_alterPaymentProcessorParams($instance, $params, &$creditCardOptions){
   \WPReporting()->listen('cartable');
  if($instance instanceof CRM_Core_Payment_OmnipayMultiProcessor ) {
    $payment_processor_id = $params['payment_processor_id'];

    $payment_processor_types = civicrm_api4('PaymentProcessor', 'get', [
      'select' => [
        'payment_processor_type_id.name',
      ],
      'where' => [
        ['id', '=', $payment_processor_id],
      ],
      'limit' => 1,
      'checkPermissions' => false,
    ]);
    $payment_processor_type_name = $payment_processor_types->first()['payment_processor_type_id.name'];

    // Custom data tranformation for recurring contributions in SystemPay
    // We consider that this function catches the hook BEFORE OmnipayPaymentProcessor 
    if($payment_processor_type_name === 'omnipay_SystemPay' && isset($creditCardOptions['token']) && $creditCardOptions['token']){
        $PaymentMethodField = cartable_get_custom_field_id('PaymentMethod', 'Parametres');
    
        if($PaymentMethodField && isset($params[$PaymentMethodField])){
            $custom_frequenty = $params[$PaymentMethodField];
            $creditCardOptions['token']['frequencyUnit'] = str_replace(['ly', '_apportionment'], '', $custom_frequenty);
            // Set default values in case of corrupted data
            if(empty($creditCardOptions['token']['frequencyInterval'])){
                $creditCardOptions['token']['frequencyInterval'] = 1;
            }
            if(empty($creditCardOptions['token']['frequencyUnit'])){
                $creditCardOptions['token']['frequencyUnit'] = 'month';
            }

            $rrule = cartable_generateRrule(date('Y-m-d'), strtoupper($creditCardOptions['token']['frequencyUnit']).'LY', $creditCardOptions['token']['frequencyInterval']);
            $creditCardOptions['token']['vads_sub_desc'] = $rrule;

            $MembershipToComplete_id = cartable_get_membership_to_complete_id();
            if($MembershipToComplete_id){
                $MembershipFirstAmount = cartable_get_membership_first_amount();
                $creditCardOptions['cancelUrl'].="&membership-to-complete={$MembershipToComplete_id}&membership-first-amount={$MembershipFirstAmount}";
                $creditCardOptions['errorUrl'].="&membership-to-complete={$MembershipToComplete_id}&membership-first-amount={$MembershipFirstAmount}";

                if($MembershipFirstAmount ){
                    $creditCardOptions['token']['token']['vads_sub_init_amount'] = $MembershipFirstAmount*100;
                    $creditCardOptions['token']['token']['vads_sub_init_amount_number'] = 1;
                }
            }
        }
    }
  }
  \WPReporting()->stop();
}

/**
 * Implements hook_civicrm_buildForm().
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_buildForm/
 */
function cartable_civicrm_buildForm($formName, &$form) {
  \WPReporting()->listen('cartable');
  header('FormName:'.$formName);
  $templatePath = realpath(dirname(__FILE__)."/templates");
  CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/cartable.js');

  if(is_admin()){
      CRM_Core_Resources::singleton()->addStyleFile('cartable', 'assets/built/admin.css');
  }
  if($formName == 'CRM_Contribute_Form_Search'
  || $formName == 'CRM_Pledge_Form_Search'){
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/contribution-counter.js');
  }

  if ( in_array($formName, [
        'CRM_Member_Form_Membership',
        'CRM_Custom_Form_CustomDataByType',
    ])
  ) {
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/membership-admin.js');
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/membership.vars.tpl"]);
  }
  if ($formName == 'CRM_Financial_Form_Payment') {
    if(false != $processor_id = filter_input(INPUT_GET, 'processor_id', FILTER_SANITIZE_NUMBER_INT)){
        $paymentProcessors = civicrm_api4('PaymentProcessor', 'get', [
            'checkPermissions' => false,
            'where' => [
                ['id', '=', $processor_id],
            ],
        ]);
        if(in_array($paymentProcessors[0]['class_name'], ['Payment_SDD', 'Payment_SDDNG'])){
            CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/sepa-fill-form.js');
            CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/my-data.vars.tpl"]);
        }
    }
  }
  if ($formName == 'CRM_Contribute_Form_Contribution_Main') {
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'vendor/agencenous/jquery-tabs/jquery-tabs.js');
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/academies.js');
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/membership.js');
    CRM_Core_Resources::singleton()->addStyleFile('cartable', 'vendor/agencenous/jquery-tabs/jquery-tabs.css');
    CRM_Core_Resources::singleton()->addStyleFile('cartable', 'assets/built/academies.css');
    CRM_Core_Resources::singleton()->addStyleFile('cartable', 'assets/built/membership.css');
    CRM_Core_Resources::singleton()->addStyleFile('cartable', 'assets/built/profile.css');
  }
  if ( in_array($formName, [
            'CRM_Activity_Form_ActivityLinks',
            ])
    ){
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/reset-password.js');
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/reset-password.vars.tpl"]);
  }
  if ( in_array($formName, [
        'CRM_Contribute_Form_Contribution_Main',
        'CRM_Contact_Form_Contact',
        'CRM_Contact_Form_Inline_CustomData',
        'CRM_Profile_Form_Edit',
    ])
  ) {
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/donnees-particulier.js');
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/donnees-particuliers.vars.tpl"]);
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/syndicates.vars.tpl"]);
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/my-data.vars.tpl"]);
}
    if (
        in_array($formName, [
            'CRM_Contact_Form_Contact',
            'CRM_Contact_Form_Inline_ContactInfo',
            'CRM_Contactlayout_Form_Inline_ProfileBlock',
            'CRM_Profile_Form_Edit',
            'CRM_Contribute_Form_Contribution_Main'
        ])
    ) {
        CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/form-constrained-fields.js');
        CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/form-constrained-fields.vars.tpl"]);
    }
if($formName == 'CRM_Contact_Form_Search_Advanced') {
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/search-custom.js');
  }
  if ( in_array($formName, [
        'CRM_Contact_Form_Contact',
        'CRM_Contact_Form_Inline_CustomData',
    ])
    ){
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/donnees-syndicat.js');
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/donnees-syndicats.vars.tpl"]);
  }
  if ( in_array($formName, [
        'CRM_Contribute_Form_Contribution_Confirm',
        'CRM_Contribute_Form_Contribution_ThankYou',
    ])
  ){
    CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/contribution-confirm.js');
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/syndicates.vars.tpl"]);
    CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/membership.vars.tpl"]);
  }

  if ($formName == 'CRM_Contribute_Form_Contribution_Main') {
        wp_enqueue_script('jquery');
        // CRM_Core_Resources::singleton()->addStyleFile('cartable', 'assets/built/contribution-pages.css');
        CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/contribution-pages.js');
  }
    if(in_array($formName, [
        'CRM_Contact_Form_Search_Advanced',
        'CRM_Contact_Form_Search_Basic',
        'CRM_Contribute_Form_Contribution',
        'CRM_Member_Form_Membership'
    ])) {
        CRM_Core_Resources::singleton()->addScriptFile('cartable', 'assets/built/search-confirm-submit.js');
        CRM_Core_Region::instance('page-body')->add(['template' => "CRM/Cartable/search-confirm.vars.tpl"]);
    }

    $MembershipToComplete_id = cartable_get_membership_to_complete_id();
    if($MembershipToComplete_id){
        $form->addElement(new \HTML_QuickForm_hidden('membership-to-complete', $MembershipToComplete_id));
    }
    $MembershipFirstAmount = cartable_get_membership_first_amount();
    if($MembershipFirstAmount){
        $form->addElement(new \HTML_QuickForm_hidden('membership-first-amount', $MembershipFirstAmount));
    }
    \WPReporting()->stop();
}

/**
 * Implements hook_civicrm_alterCustomFieldDisplayValue().
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_alterCustomFieldDisplayValue/
 */
function cartable_civicrm_alterCustomFieldDisplayValue(&$displayValue, $value, $entityId, $fieldInfo) {
    \WPReporting()->listen('cartable');
    if ($fieldInfo['name'] == 'PaymentProcessors') {
        $pps = [];
        $paymentProcessors = \Civi\Api4\PaymentProcessor::get(false)
        ->addWhere('is_test', 'IN', [TRUE, false])
        ->execute();
        $pp = [];
        $selectecValues = explode(',', $displayValue);
        foreach ($paymentProcessors as $paymentProcessor) {
            $pp[$paymentProcessor['id']] = $paymentProcessor;
        }
        foreach($selectecValues as $p){
            if(isset($pp[$p])){
                $pps[] = '<span>'.$pp[$p]['title'].($pp[$p]['is_test'] == '1' ? ' <em>(test)</em>' : '').'</span>';
            }
        }
        $displayValue = implode(', ', $pps);
    }
    if (is_admin() && $fieldInfo['name'] == 'GrilleCotisation') {
        $rows = explode("\n", strip_tags($displayValue));
        if(empty($rows)){
            $rows = explode("\n", civicrm_api3('Setting', 'getvalue', [
                'return' => "value",
                'name' => "cartable_membership_rates",
            ]));
        }
        if(count($rows)){
            $html='
            <table class="cartable-edit-grillecotisation-preview">
                <thead>
                    <tr><th>'.ts('Label').'</th><th>'.ts('Amount').'</th></tr>
                </thead>
                <tbody>';
            foreach($rows as $row){
                if(empty($row)) {
                    continue;
                }
                $cols = explode('|', $row);
                $html.='
                    <tr><td>'.trim($cols[0]).'</td><td>'.trim($cols[1]).'€</td></tr>';
            }
            $html.='
                </tbody>
            </table>';
            $displayValue = $html;
        }
    }
    if ($fieldInfo['name'] == 'GrilleSalariale') {
        $data = explode('|', $displayValue);
        if(count($data)==2){
            list($SyndicId, $Salary) = $data;

            $GrilleCotisationField = cartable_get_custom_field_id('GrilleCotisation', 'ParametresAdhesion');
            $GrilleCotisation = civicrm_api3('Contact', 'getvalue', ['return'=>$GrilleCotisationField, 'id'=>$SyndicId]);
            if(!$GrilleCotisation){
                $GrilleCotisation = civicrm_api3('Setting', 'getvalue', [
                    'return' => "value",
                    'name' => "cartable_membership_rates",
                ]);
            }
            $membershipRates = cartable_parse_rates($GrilleCotisation);
            if(isset($membershipRates[$Salary])){
                $displayValue = $membershipRates[$Salary];
                return;
            }

        }
    }
    if ($fieldInfo['name'] == 'PaymentMethod') {
        $FrequenciesData = cartable_get_values('payment-methods', false);
        if(isset($FrequenciesData[$displayValue])){
            $displayValue = $FrequenciesData[$displayValue]['title'];
        }
    }
    if($fieldInfo['name'] == 'MembershipStatus'){
        $contactId = 0;
        if (isset($_GET['cid'])) {
            $contactId = intval($_GET['cid']);
        }
        $memberships = civicrm_api3('Membership', 'get', [
            'contact_id' => $contactId,
        ]);
        if (!empty($memberships['values'])) {
            $membership = reset($memberships['values']);
            $membershipStatusId = $membership['status_id'];
            $membershipStatus = civicrm_api3('MembershipStatus', 'getvalue', [
                'return' => 'name',
                'id' => $membershipStatusId,
            ]);
            $membershipStatus = ts($membershipStatus);
        } else {
            $membershipStatus = E::ts('No memberships found for this contact');
        }
        $displayValue = $membershipStatus;
    }
    \WPReporting()->stop();
}


/**
 * Add token services to the container.
 * Implements hook_civicrm_container().
 *
 * @param \Symfony\Component\DependencyInjection\ContainerBuilder $container
 */
function cartable_civicrm_container(ContainerBuilder $container)
{
    $container->addResource(new FileResource(__FILE__));
    $container->findDefinition('dispatcher')->addMethodCall(
        'addListener',
        ['civi.token.list', 'cartable_register_tokens']
    )->setPublic(true);
    $container->findDefinition('dispatcher')->addMethodCall(
        'addListener',
        ['civi.token.eval', 'cartable_evaluate_tokens']
    )->setPublic(true);
}

/**
 * Register tokens.
 * 
 * @see https://docs.civicrm.org/dev/en/latest/framework/token/
 */
function cartable_register_tokens(\Civi\Token\Event\TokenRegisterEvent $e)
{
    $e->entity('SUD')
      ->register('last_sepa_link', E::ts('Link of the last SEPA mandate'))
      ->register('thank_you_syndicate', E::ts('Thank you message from the syndicate'))
      ->register('syndicate_name', E::ts('Name of the syndicate'))
      ->register('contribution_url', E::ts('URL of the contribution'))
    ;
}

/**
 * Evaluate tokens.
 * 
 * @see https://docs.civicrm.org/dev/en/latest/framework/token/
 */
function cartable_evaluate_tokens(\Civi\Token\Event\TokenValueEvent $e)
{
    $usedTokens = $e->getTokenProcessor()->getMessageTokens();
    if(empty($usedTokens['SUD'])){
        return;
    }

    // Compute contact_ids
    $contacts = [];
    foreach ($e->getRows() as $row) {
        $contacts[$row->context['contactId']] = [];
    }
    $cids = array_keys($contacts);

    // Populate data for all syndicates
    if(
        in_array('syndicate_name', $usedTokens['SUD'])
        ||
        in_array('thank_you_syndicate', $usedTokens['SUD'])
    ){
        $indexed_syndicates = [
            0 => [
                'name' => E::ts('SUD Education'),
                'thank_you' => E::ts('Thank you for your contribution.'),
            ]
        ];
        $syndicates = \Civi\Api4\Contact::get(false)
            ->addSelect('display_name')
            ->addSelect('ParametresAdhesion.ThankYouMessage')
            ->addWhere('contact_sub_type', '=', 'SyndicatSUD')
            ->execute();
        foreach ($syndicates as $syndicate) {
            $indexed_syndicates[$syndicate['id']] = [
                'name' => $syndicate['display_name'],
                'thank_you' => $syndicate['ParametresAdhesion.ThankYouMessage'],
            ];
        }
        // Civi::log()->debug('Cartable: indexed_syndicates tokens ', $indexed_syndicates);


        // Populate syndicate from relation
        foreach($cids as $cid){
            $contacts[$cid]['syndicate_name'] = $indexed_syndicates[0]['name'];
            $contacts[$cid]['thank_you_syndicate'] = $indexed_syndicates[0]['thank_you'];
            $MaybeSyndicates = cartable_get_syndicates_of($cid, 'member');
            if(!empty($MaybeSyndicates) && isset($indexed_syndicates[$MaybeSyndicates[0]]) && !empty($indexed_syndicates[$MaybeSyndicates[0]]['name'])){
                $contacts[$cid]['syndicate_name'] = $indexed_syndicates[$MaybeSyndicates[0]]['name'];
            }
            if(!empty($MaybeSyndicates) && isset($indexed_syndicates[$MaybeSyndicates[0]]) && !empty($indexed_syndicates[$MaybeSyndicates[0]]['thank_you'])){
                $contacts[$cid]['thank_you_syndicate'] = $indexed_syndicates[$MaybeSyndicates[0]]['thank_you'];
            }
        }

        // Populate syndicate from last membership
        $indexed_memberships = [];
        $memberships = \Civi\Api4\Membership::get(false)
            ->addSelect('contact_id', 'MAX(id) AS m_id')
            ->addGroupBy('contact_id')
            ->addWhere('contact_id', 'IN', $cids)
            ->addChain('_membership', \Civi\Api4\Membership::get(false)
                ->addSelect('Parametres.GrilleSalariale', 'contact_id', 'id', 'join_date')
                ->addWhere('id', '=', '$m_id')
            )
            ->execute();
        foreach ($memberships as $_membership) {
            $membership = $_membership['_membership'];
            if(isset($membership['contact_id'])){
                $indexed_memberships[$membership['contact_id']] = $membership;
            }
            if(isset($membership['Parametres.GrilleSalariale'])){
                $data = explode('|', $membership['Parametres.GrilleSalariale']);
                if(count($data)==2){
                    list($SyndicId, $Salary) = $data;
                    $contacts[$membership['contact_id']]['syndicate_name'] = isset($indexed_syndicates[$SyndicId]) ? $indexed_syndicates[$SyndicId]['name'] : $ThankYouMessages[0]['name'];
                    $contacts[$membership['contact_id']]['thank_you_syndicate'] = isset($indexed_syndicates[$SyndicId]) ? $indexed_syndicates[$SyndicId]['thank_you'] : $ThankYouMessages[0]['thank_you'];
                }
            }
        }
    }
    
    // FIND contribution URL
    if(
        in_array('contribution_url', $usedTokens['SUD'])
    ){
        $profileUrl = civicrm_api4('Setting', 'get', ['select' => ['cartable_profile_url',], 'checkPermissions' => false]);
        foreach($cids as $cid){
            $contacts[$cid]['contribution_url'] = $profileUrl[0]['value'];
        }
    }
    
    // FIND most recent SEPA Mandates (per contact)
    if(
        in_array('last_sepa_link', $usedTokens['SUD'])
    ){
        $mandates = \Civi\Api4\SepaMandate::get(false)
            ->addSelect('contact_id', 'MAX(id) AS m_id')
            ->addGroupBy('contact_id')
            ->addWhere('contact_id', 'IN', $cids)
            ->addChain('_mandate', \Civi\Api4\SepaMandate::get(false)
                ->addWhere('id', '=', '$m_id')
            )
            ->execute();
        foreach ($mandates as $_mandate) {
            // load the mandate
            $mandate = $_mandate['_mandate'];
            if(!isset($indexed_memberships[$mandate['contact_id']])){
                error_log('DEBUG: no membership found for contact '.$mandate['contact_id']);
                continue;
            }

            $membership = $indexed_memberships[$mandate['contact_id']];
            if($membership['join_date'] != substr($mandate['creation_date'], 0, 10)){
                error_log('DEBUG: Mandate creation date differs from membership join date');
                continue;
            }

            // make sure there is an array
            if (!isset($values[$mandate['contact_id']])) {
                $values[$mandate['contact_id']] = array();
            }
            $contacts[$mandate['contact_id']]['last_sepa_link']  = '<a href="'.CRM_Utils_System::url('civicrm/sud/mandate', "m_id={$mandate['id']}", true).'">'.E::ts('Download my SEPA Mandate').'</a>';
        }
    }

    Civi::log()->debug('Cartable: contacts tokens ', $contacts);


    foreach ($e->getRows() as $row) {
        /** @var TokenRow $row */
        $contact_id = $row->context['contactId'];
        // $row->format('text/html');
        foreach($usedTokens['SUD'] as $token){
            if(isset($contacts[$contact_id][$token])){
                $row->tokens('SUD', $token, $contacts[$contact_id][$token]);
            }
            else{
                $row->tokens('SUD', $token, 'UNFOUND TOKEN:'.$token.' FOR CONTACT:'.$contact_id);
            }
        }
    }
}

/**
 * Set visibility to Managers on Groups created by a Syndicate
 * @see https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_aclGroup/
 */
function cartable_civicrm_aclGroup($type, $contactID, $tableName, &$allGroups, &$currentGroups){
    \WPReporting()->listen('cartable');

    if(null === $contactID){
        \WPReporting()->stop();
        return;
    }

    // Give visibility to groups created by the syndicate managed by the current contact
    $syndicates_ids = cartable_get_syndicates_of($contactID, ['manager', 'team']);
    $groups = civicrm_api4('Group', 'get', [
    'where' => [
        ['created_id', 'IN', $syndicates_ids],
    ],
    'checkPermissions' => false,
    ]);
    foreach($groups as $group){
        $currentGroups[] = $group['id'];
    }
    $currentGroups = array_unique($currentGroups);
    
    // Make children groups visible too
    $children_groups = civicrm_api4('Group', 'get', [
        'select' => [
            'id',
        ],
        'where' => [
            ['parents', 'IN', $currentGroups],
        ],
        'limit' => 25,
        'checkPermissions' => false,
    ]);
    foreach($children_groups as $group){
        $currentGroups[] = $group['id'];
    }
    $currentGroups = array_unique($currentGroups);
    \WPReporting()->stop();
}

/**
 * Implements hook_civicrm_permission_check().
 * @link https://docs.civicrm.org/dev/en/latest/hooks/hook_civicrm_permission_check/
 */
function cartable_civicrm_permission_check($permission, &$granted) {
    \WPReporting()->listen('cartable');
    // List of args to look for
    $args_lookup = [
        "civicrm/ajax/api4/Contact/updatePassword",
    ];

    // get caller function and try to determin context
    $backtrace = debug_backtrace();
    $caller = false;
    foreach($backtrace as $index=>$trace){
        if(isset($trace['args'][0]) && is_array($trace['args'][0]) ){
            foreach($args_lookup as $arg){
                if( $trace['args'][0] === explode('/', $arg)){
                    $caller = $arg;
                    break;
                }
            }
        }
    }
    if(!$caller){
        \WPReporting()->stop();
        return;
    }
    
    // api4/contact/updatePassword context
    if($caller === "civicrm/ajax/api4/Contact/updatePassword"){
        $current_user = wp_get_current_user();
        if(user_is_admin($current_user)){
            \WPReporting()->stop();
            return;
        }

        if(!isset($_POST['params'])){
            \WPReporting()->stop();
            return;
        }
        
        $params = is_array($_POST['params']) ? $_POST['params'] : json_decode($_POST['params'], true);
        if(!isset($params['userID'])){
            \WPReporting()->stop();
            return;
        }
        $contactID = $params['userID'];
        
        $civiID = CRM_Core_Session::singleton()->getLoggedInContactID();
        $granted = user_is_manager_of($civiID, $contactID);
    }
    \WPReporting()->stop();
}

/**
 * Add submenu in 'Action' menu
 * @param mixed $actions
 * @param mixed $contactID
 * @return void
 */
function cartable_civicrm_summaryActions(&$actions, $contactID)
{
    if ( is_user_logged_in() ) {
      
        //current
        $current_user = wp_get_current_user();
        $civiID = CRM_Core_Session::singleton()->getLoggedInContactID();

        $admin = user_is_admin($current_user);
        $is_manager = user_is_manager_of($civiID, $contactID);
        if($admin || $is_manager) {
            if ($contactID) {
                $actions['otherActions']['reset_password'] = array(
                    'title' => E::ts('Reset password'),
                    'weight' => 999,
                    'ref' => 'cartable_reset_password',
                    'key' => 'sud',
                    'href' => 'javascript:cartable_reset_password(' . $contactID . ');',
                    'class' => 'no-popup',
                    'icon' => 'fa fa-key'
                );
            }
        }
    }
}
  
/**
 * Discriminate between native Systempay subscriptions and legacy Omnipay contributions that have been migrated
 * 
 * @param string $orderId
 * @param array $contributionRecur
 * @param array $params
 * 
 * @return void
 */
function cartable_civicrm_systempay_checktxn_orderid(&$orderId, $contributionRecur, $params){
    // Get the first contribution of this recurring contribution where the trxn_id is not like %-%
    Civi::log()->debug("Cartable: checktxn_orderid: {$orderId}", $contributionRecur);
    $contribution = civicrm_api4('Contribution', 'get', [
        'select' => [
            'id',
            'source',
            'trxn_id',
            'receive_date',
        ],
        'where' => [
            ['contribution_recur_id', '=', $contributionRecur['id']],
        ],
        'orderBy' => [
            'receive_date' => 'ASC',
        ],
        'limit' => 1,
        'checkPermissions' => false,
    ]);
    foreach($contribution as $c){
        // If the contribution is older than the migration date, we consider it as a legacy contribution
        if(strtotime($c['receive_date']) < strtotime('2024-11-06')){
            $orderId = str_pad($c['id'], 6, '0', STR_PAD_LEFT);
            Civi::log()->debug("Cartable: found legacy Omnipay Contribution. Alter Systempay orderId: {$orderId}", $c);
        }
    }
}

/**
 * Implements hook_cdntaxreceipts_writeReceipt.
 */
function cartable_cdntaxreceipts_writeReceipt(&$pdf, $pdf_variables, $receipt) {
    // Extract variables
    $mode = $pdf_variables["mode"];
    $mymargin_left = $pdf_variables["mymargin_left"];
    $mymargin_top = $pdf_variables["mymargin_top"];
    $is_duplicate = $pdf_variables["is_duplicate"];
    $pdf_img_files_path = $pdf_variables["pdf_img_files_path"];
    $line_1 = $pdf_variables["line_1"];
    $source_funds = $pdf_variables["source_funds"];
    $amount = $pdf_variables["amount"];
    $display_date = $pdf_variables["display_date"];
    $issued_on = $pdf_variables["issued_on"];
    $location_issued = $pdf_variables["location_issued"];
    $receipt_number = $pdf_variables["receipt_number"];
    $displayname = $pdf_variables["displayname"];
    $address_line_1 = $pdf_variables["address_line_1"];
    $address_line_1b = $pdf_variables["address_line_1b"];
    $address_line_2 = $pdf_variables["address_line_2"];
    $address_line_3 = $pdf_variables["address_line_3"];
    $inkind_values = $pdf_variables["inkind_values"];
    $display_year = $pdf_variables["display_year"];
    $issue_type = $pdf_variables["issue_type"];
    $receipt_contributions = $pdf_variables['receipt_contributions'];
    $receipt_status = $pdf_variables['receipt_status'];

    // Sud variables
    $cid = CRM_Core_Session::singleton()->getLoggedInContactID();
    $syndicateId = cartable_get_syndicates_of($cid, 'member');
    $syndicate = civicrm_api4('Contact', 'get', [
        'select' => [
          'display_name',
          'address.street_address',
          'address.postal_code',
          'address.city',
        ],
        'join' => [
          ['Address AS address', 'LEFT'],
        ],
        'where' => [
          ['contact_type', '=', 'Organization'],
          ['id', '=', $syndicateId[0]],
        ],
        'limit' => 1,
        'checkPermissions' => FALSE,
    ]);


    $totalLetter = cartable_numberToWords($amount);
    $date = new DateTime($display_date);
    $formatted_date = $date->format('d/m/Y');

    // Middle center section
    $pdf->SetFont('Helvetica', 'B', 10);
    $pdf->Write(10, ts('Receipt for tax deduction', array('domaine' => 'cartable')), '', 0, 'C', TRUE, 0, FALSE, FALSE, 0);
    $pdf->SetY($mymargin_top + 4);
    $pdf->SetFont('Helvetica', '', 8);
    $pdf->Write(10, ts('Articles 200-5, 238 bis and 885-0 V bis A', array('domaine' => 'cartable')), '', 0, 'C', TRUE, 0, FALSE, FALSE, 0);
    $pdf->SetY($mymargin_top + 7);
    $pdf->Write(10, ts('of the General Tax Code', array('domaine' => 'cartable')), '', 0, 'C', TRUE, 0, FALSE, FALSE, 0);
    
    if ($mode == CDNTAXRECEIPTS_MODE_PREVIEW) {
        $pdf->Image($pdf_img_files_path . 'preview_mode.png', $mymargin_left + 55, $mymargin_top + 30, '', 45);
    }
    else if ($receipt_status == 'cancelled') {
        $pdf->Image($pdf_img_files_path . 'cancelled_trans.png', $mymargin_left + 55, $mymargin_top + 30, '', 45);
    }
    else if ($is_duplicate) {
        $pdf->Image($pdf_img_files_path . 'duplicate_trans.png', $mymargin_left + 55, $mymargin_top + 30, '', 45);
    }
    // Top left section
    $pdf_template_file = _cdntaxreceipts_get_image_filepath('receipt_pdftemplate');
    if (empty($pdf_template_file)) {
        $receipt_logo = _cdntaxreceipts_get_image_filepath('receipt_logo');
        if ( empty($receipt_logo) ) {
        $receipt_logo = $pdf_img_files_path . 'your-logo.png';
        }
        $pdf->Image($receipt_logo, $mymargin_left, $mymargin_top, '', 30);
        // Top right section
        $pdf->SetFont('Helvetica', 'B', 8);
        $pdf->SetY($mymargin_top);
        $pdf->Write(10, ts('Receipt: %1', array(1 => $receipt_number, 'domain' => 'cartable')), '', 0, 'R', TRUE, 0, FALSE, FALSE, 0);
        $pdf->SetY($mymargin_top + 4);
        $pdf->SetFont('Helvetica', 'B', 8);
        $pdf->Write(10, ts('Donor: %1', array(1 => $displayname,'domaine' => 'cartable')), '', 0, 'R', TRUE, 0, FALSE, FALSE, 0);
        $pdf->SetY($mymargin_top + 8);
        $pdf->SetFont('Helvetica', 'B', 8);
        $pdf->Write(10, $address_line_1, '', 0, 'R', TRUE, 0, FALSE, FALSE, 0);
        $pdf->SetY($mymargin_top + 12);
        $pdf->SetFont('Helvetica', 'B', 8);
        $pdf->Write(10, $address_line_2, '', 0, 'R', TRUE, 0, FALSE, FALSE, 0);
        $pdf->SetY($mymargin_top + 16);
    }

    // Right section
    $x_detailscolumn = 15;
    $y_detailscolumnstart = 45;
    $pdf_template_file = _cdntaxreceipts_get_image_filepath('receipt_pdftemplate');
    if (empty($pdf_template_file)) {
        $background_image = _cdntaxreceipts_get_image_filepath('receipt_watermark');
        if ($background_image) {
        $pdf->Image($background_image, $mymargin_left + $x_detailscolumn, $mymargin_top + $y_detailscolumnstart + 6, '', 40);
        }
    }
    $pdf->SetXY($mymargin_left + $x_detailscolumn, $mymargin_top + $y_detailscolumnstart + 6);
    $pdf->SetFont('Helvetica', '', 8);
    $pdf->Write(8, ts("The %1 association recognizes that donations received, in one or more payments, during the year %2, for the total sum of: %3, or %4, are eligible for the tax reduction provided for in article 200 of the CGI (Individuals) of the General Tax Code.", 
        array(
            1 => $syndicate[0]['display_name'] ?: Civi::settings()->get('org_name'),
            2 => $display_year,
            3 => number_format($amount, 2) . "€",
            4 => $totalLetter . " euros",
            'domain' => 'cartable'
        )
    ));
    
    // Left section
    $pdf->SetFont('Helvetica', 'B', 8);
    $pdf->SetY($mymargin_top + 28);
    $pdf->Write(10, ts("Recipient of the donation:", array('domain' => 'cartable')), '', 0, 'L', TRUE, 0, FALSE, FALSE, 0);
    $pdf->SetFont('Helvetica', 'B', 8);
    $pdf->SetY($mymargin_top + 32);
    $pdf->Write(10, $syndicate[0]['display_name'] ?: Civi::settings()->get('org_name'), '', 0, 'L', TRUE, 0, FALSE, FALSE, 0);
    $pdf->SetY($mymargin_top + 36);
    $pdf->Write(10, $syndicate[0]['address.street_address'] ? $syndicate[0]['address.street_address'] : Civi::settings()->get('org_address_line1'), '', 0, 'L', TRUE, 0, FALSE, FALSE, 0);
    $pdf->SetY($mymargin_top + 40);
    $pdf->Write(10, $syndicate[0]['address.postal_code'] && $syndicate[0]['address.city'] ? $syndicate[0]['address.postal_code'] . ' '. $syndicate[0]['address.city'] : Civi::settings()->get('org_address_line2'), '', 0, 'L', TRUE, 0, FALSE, FALSE, 0);

    //Bottom left section
    $pdf->SetFont('Helvetica', '', 8);
    $pdf->SetY($mymargin_top + 80);
    $pdf->Write(10, ts("The: %1", array(1 => $formatted_date, 'domain' => 'cartable')),'', 0, 'L', TRUE, 0, FALSE, FALSE, 0);

    // Bottom right section
    $pdf->SetFont('Helvetica', '', 8);
    $pdf->SetY($mymargin_top + 80);
    $pdf->Write(10, ts('According to Cerfa model no. 11580-03', array('domain' => 'cartable')),'', 0, 'R', TRUE, 0, FALSE, FALSE, 0);
    // }

    // Line at the bottom
    $pdf->Line($mymargin_left, $mymargin_top + 90, 198, $mymargin_top + 90, 'dash');


    return array(TRUE);
}

function cartable_numberToWords($number) {
    $f = new NumberFormatter("fr", NumberFormatter::SPELLOUT);
    return ucfirst($f->format($number));
}