<?php
use CRM_Cartable_ExtensionUtil as E;

function cartable_get_dir_path($path){
    $complete_path = CARTABLE_DIR.'/'.$path;
    if(is_dir($complete_path)){
        return $complete_path;
    }
    return false;
}

function cartable_get_file_path($path){
    $complete_path = CARTABLE_DIR.'/'.$path;
    if(file_exists($complete_path)){
        return $complete_path;
    }
    return false;
}

function cartable_get_file_content($path){
    $complete_path = cartable_get_file_path($path);
    if($complete_path){
        return file_get_contents($complete_path);
    }
    return null;
}

function cartable_include($path){
    $complete_path = cartable_get_file_path($path);
    if($complete_path){
        include($complete_path);
        return $complete_path;
    }
    return false;
}

/**
 * Check if current_user is admin
 * @param mixed $current_user
 * @return bool
 */
function user_is_admin($current_user) {
    if (current_user_can('manage_settings') || is_super_admin()) {
        $admin = true;
    } else {
        $admin = false;
    }
    return $admin;
}


/**
 * Get mapped roles
 * @return array
 */
function cartable_roles(){
    $roles = [
        'manager' => 'SUD_Gestionnaire',
        'team' => 'SUD_Equipiere_De',
        'member' => 'SUD_Membre_De',
        'contact' => 'SUD_Contact_De',
    ];
    return $roles;
}

/**
 * Get the IDs of the syndicates managed by the contact
 * @param int $contact_id Id of the contact
 * @param string $roles array of Role of the syndicates. must be one of cartable_roles(). Default is any.
 * @return array of int
 */
function cartable_get_syndicates_of(int $contact_id, $roles=null){
    \WPReporting()->listen('cartable');

    $available_roles = cartable_roles();

    if(is_string($roles)){
        $roles = [$roles];
    }
    if(null === $roles){
        $roles = array_keys($available_roles);
    }
    $roles_in = [];
    foreach($roles as $role){
        if(!isset($available_roles[$role])){
            throw new \Exception("Unknown role $role");
        }
        $roles_in[] = $available_roles[$role];
    }
    
    $syndicate_ids = [];
    $relationships = civicrm_api4('Relationship', 'get', [
        'select' => [
            'contact_id_b',
        ],
        'where' => [
            ['relationship_type_id:name', 'IN', $roles_in],
            ['contact_id_a', '=', $contact_id],
        ],
        'checkPermissions' => false,
    ]);
    foreach($relationships as $relationship){
        $syndicate_ids[] = $relationship['contact_id_b'];
    }

    \WPReporting()->stop();
    return $syndicate_ids;
}



/**
 * Get the IDs of the members of the syndicate
 * @param int $syndicate_id Id of the syndicate
 * @param $roles array of Role of the members. must be one of cartable_roles(). Default is any.
 * @return array of int
 */
function cartable_get_contacts_of(int $syndicate_id, $roles=null){
    \WPReporting()->listen('cartable');

    $available_roles = cartable_roles();

    if(is_string($roles)){
        $roles = [$roles];
    }
    if(null === $roles){
        $roles = array_keys($available_roles);
    }
    $roles_in = [];
    foreach($roles as $role){
        if(!isset($available_roles[$role])){
            throw new \Exception("Unknown role $role");
        }
        $roles_in[] = $available_roles[$role];
    }

    $member_ids = [];
    $relationships = civicrm_api4('Relationship', 'get', [
        'select' => [
            'contact_id_a',
        ],
        'where' => [
            ['relationship_type_id:name', 'IN', $roles_in],
            ['contact_id_b', '=', $syndicate_id],
        ],
        'checkPermissions' => false,
    ]);
    foreach($relationships as $relationship){
        $member_ids[] = $relationship['contact_id_a'];
    }

    \WPReporting()->stop();
    return $member_ids;
}

/**
 * Find out if a user is the group manager of the consulted contact
 * @param mixed $manager_id Id of user
 * @param mixed $contact_id Id of consulted contact
 * @return bool
 */
function user_is_manager_of($manager_id, $contact_id){
    \WPReporting()->listen('cartable');

    $is_manager_of = false;

    if(CRM_Core_Permission::check('edit all contacts')){
        $is_manager_of = true;
    }
    else{
        $syndicates = cartable_get_syndicates_of($manager_id, ['manager', 'team']);

        foreach($syndicates as $syndicate_id){
            $members = cartable_get_contacts_of($syndicate_id, 'member');
            if(in_array($contact_id, $members)){
                $is_manager_of = true;
                break;
            }
        }
    }
   
    \WPReporting()->stop();
    return  $is_manager_of;
}


/**
 * get group id from civiId
 * @param mixed $civiID
 * @return mixed
 */
function user_group_ids($civiID) {
    $user_group_ids = array();
    if($civiID) {
        $civicrm_relationship_dao = CRM_Core_DAO::executeQuery("SELECT contact_id_b FROM civicrm_relationship WHERE contact_id_a={$civiID}");
        $array_user_groups = $civicrm_relationship_dao->fetchAll();
        foreach ($array_user_groups as $user_group) {
            $user_group_ids[] = isset($user_group['contact_id_b']) ? $user_group['contact_id_b'] : null;
        }
    }
    return  $user_group_ids;
}

// function add_cartable_group_id_meta($attachment_id) {
//     $current_user = wp_get_current_user();
//     $admin = user_is_admin($current_user);
//     if ( ! get_post_meta( $attachment_id, 'cartable_group_id', true ) ) {
//         $civiID = CRM_Core_Session::singleton()->getLoggedInContactID();
//         if($admin) {
//             $group_id = 0;
//             update_post_meta($attachment_id, 'cartable_admin_upload', 'true');
//         } else {
//             $group_id = user_group_id($civiID);
//         }
//         update_post_meta($attachment_id, 'cartable_group_id', $group_id);
//     }
// }
// add_action('add_attachment', 'add_cartable_group_id_meta');

// function restrict_gallery_by_group($where) {
//     \WPReporting()->listen('cartable', E_ALL);
//     global $wpdb;
//     if ( is_user_logged_in() ) {
//         $current_user = wp_get_current_user();
//         $admin = user_is_admin($current_user);

//         $civiID = CRM_Core_Session::singleton()->getLoggedInContactID();
        
//         if (!$admin) {
//             $group_id = user_group_id($civiID);
//             $group_id = intval($group_id);

//             $where .= " AND ($wpdb->posts.ID IN (
//                 SELECT post_id FROM $wpdb->postmeta 
//                 WHERE meta_key = 'cartable_group_id' AND meta_value = $group_id
//             ) OR $wpdb->posts.ID IN (
//                 SELECT post_id FROM $wpdb->postmeta 
//                 WHERE meta_key = 'cartable_admin_upload' AND meta_value = 'true'
//             ) OR $wpdb->posts.ID NOT IN (
//                 SELECT post_id FROM $wpdb->postmeta 
//                 WHERE meta_key = 'cartable_admin_upload' OR meta_key = 'cartable_group_id'
//             )
//             )";
//         }
//     } 
//     \WPReporting()->stop();
//     return $where;
// }
// add_filter('posts_where', 'restrict_gallery_by_group', 10, 2);


function cartable_get_values($dataset, $sort=true){
    \WPReporting()->listen('cartable', E_ALL);
    $maybe_file = cartable_get_file_path("inc/values/{$dataset}.php");
    if(file_exists($maybe_file)){
        $array = include $maybe_file;
        if($sort){
            asort($array, SORT_NATURAL);
        }
        \WPReporting()->stop();
        return $array;
    }
    \WPReporting()->stop();
    return [];
}

function cartable_get_custom_field_id($fieldname, $groupname=null){
    try{
        $FieldId = civicrm_api3('CustomField', 'getvalue', ['return'=>"id", 'name'=>$fieldname, 'custom_group_id'=>$groupname]);
        if(!$FieldId){
            return false;
        }
        return "custom_{$FieldId}";
    }
    catch(\Throwable $e){
        \WPReporting()->send($e, 'cartable');
        return;
    }
    catch(\Exception $e){
        \WPReporting()->send($e, 'cartable');
        return;
    }
}

function cartable_parse_rates($rates_setting){
    \WPReporting()->listen('cartable', E_ALL);
    $Rates = [];
    if($rates_setting){
        $RatesRows = explode("\n", $rates_setting);
        foreach($RatesRows as $RateRow){
            $Rate = explode('|', $RateRow);
            $Rates[trim($Rate[0])] = (int) trim($Rate[1]);
        }
    }
    \WPReporting()->stop();
    return $Rates;
}

function cartable_get_donnees_particuliers_vars(){
    \WPReporting()->listen('cartable', E_ALL);
    $fields = include cartable_get_file_path("inc/conditionnal-fields.php");
    \WPReporting()->stop();
    return $fields;
}

function cartable_get_donnees_syndicats_vars(){
    $data = [
        'paymentProcessors' => [],
    ];

    $paymentProcessors = \Civi\Api4\PaymentProcessor::get(false)
    ->addWhere('is_test', 'IN', [TRUE, false])
    ->execute();
    foreach ($paymentProcessors as $paymentProcessor) {
        $data['paymentProcessors'][] = [
            'id' => $paymentProcessor['id'],
            'text' => $paymentProcessor['name'].($paymentProcessor['is_test'] ? ' (test)' : ''),
            'debug' => $paymentProcessor,
        ];
    }

    return $data;
}

function cartable_get_my_data(){
    \WPReporting()->listen('cartable', E_ALL);
    $current_contact_id = CRM_Core_Session::singleton()->getLoggedInContactID();
    $data = [
        'id' => $current_contact_id,
        'name' => '',
        'sepa_holder' => '',
        'sepa_bic' => '',
        'sepa_iban' => '',
        'sepa_bank' => '',
    ];
    if($current_contact_id){
        $me = civicrm_api4('Contact', 'get', [
            'select' => [
                'id', 
                'ParametresPaiement.*',
            ],
            'where' => [
                ['id', '=', $current_contact_id],
            ],
        ]);
        $syndicate_id = civicrm_api4('Relationship', 'get', [
            'select' => [
                'contact_id_b',
            ],
            'where' => [
                ['contact_id_a', '=', $current_contact_id],
                ['relationship_type_id:name', '=', 'SUD_Membre_De'],
            ],
        ]);
        if($syndicate_id && isset($syndicate_id[0])) {
            $syndicate = civicrm_api4('Contact', 'get', [
                'select' => [
                    'DonneesSyndicat.Departements',
                ],
                'where' => [
                    ['contact_sub_type', '=', 'SyndicatSUD'], 
                    ['id', '=', $syndicate_id[0]['contact_id_b']],
                ],
            ]);
        }
        if(isset($me[0])){
            $data['sepa_holder'] = $me[0]['ParametresPaiement.SEPA_Holder'];
            $data['sepa_bic'] = $me[0]['ParametresPaiement.SEPA_BIC'];
            $data['sepa_iban'] = $me[0]['ParametresPaiement.SEPA_IBAN'];
            $data['sepa_bank'] = $me[0]['ParametresPaiement.SEPA_Bank'];
        }
        if(!empty($syndicate)) {
            $data['syndicat'] = $syndicate;
        }
    }
    \WPReporting()->stop();
    return $data;
}

function cartable_get_membership_vars($vars){
    \WPReporting()->listen('cartable', E_ALL);

    $MembershipToComplete_id = null;
    $MembershipToComplete = null;
    $MembershipToComplete_id = cartable_get_membership_to_complete_id();
    if($MembershipToComplete_id){
        $MembershipToComplete = cartable_get_membership_to_complete($MembershipToComplete_id);
    }
    $MembershipFirstAmount = cartable_get_membership_first_amount();

    $GrilleSalarialeField = cartable_get_custom_field_id('GrilleSalariale', 'Parametres');
    $PaymentMethodField = cartable_get_custom_field_id('PaymentMethod', 'Parametres');
    $numberOfMonthsForContributionField = cartable_get_custom_field_id('numberOfMonthsForContribution', 'Parametres');
    $numberOfMonthsForProrataField = cartable_get_custom_field_id('numberOfMonthsForProrata', 'Parametres');
    $annualAmountForContributionField = cartable_get_custom_field_id('annualAmountForContribution', 'DonneesParticulier');
    $membershipRatio = civicrm_api3('Setting', 'getvalue', [
        'return' => "value",
        'name' => "cartable_membership_ratio",
    ]);
    $membershipRatesSettings = civicrm_api3('Setting', 'getvalue', [
        'return' => "value",
        'name' => "cartable_membership_rates",
    ]);
    $membershipTaxDeduction = civicrm_api3('Setting', 'getvalue', [
        'return' => "value",
        'name' => "cartable_tax_deduction",
    ]);
    $membershipFallbackJoinURL = civicrm_api3('Setting', 'getvalue', [
        'return' => "value",
        'name' => "cartable_fallback_join_url",
    ]);
    $membershipRates = cartable_parse_rates($membershipRatesSettings);
    $otherAmountField = null;
    $membershipField = null;
    $contributionField = null;
    if(isset($vars['priceSet']['fields'])){
        foreach($vars['priceSet']['fields'] as $amountFieldID=>$amountField){
            if($amountField['name'] === 'other_amount'){
                $otherAmountField = $amountField;
            }
            if($amountField['name'] === 'membership_amount'){
                $membershipField = $amountField;
            }
            if($amountField['name'] === 'contribution_amount'){
                $contributionField = $amountField;
            }
        }
    }

    // TODO: Make title, description and label editable in settings
    $FrequenciesData = cartable_get_values('payment-methods', false);

    $variables = [
        'membershipRatio' => $membershipRatio,
        'membershipRates' => $membershipRates,
        'membershipTaxDeduction' => $membershipTaxDeduction ?? 100,
        'membershipFallbackJoinURL' => $membershipFallbackJoinURL,
        'otherAmountField' => $otherAmountField,
        'membershipField' => $membershipField,
        'contributionField' => $contributionField,
        'GrilleSalarialeField' => $GrilleSalarialeField,
        'PaymentMethodField' => $PaymentMethodField,
        'numberOfMonthsForContributionField' => $numberOfMonthsForContributionField,
        'numberOfMonthsForProrataField' => $numberOfMonthsForProrataField,
        'annualAmountForContributionField' => $annualAmountForContributionField,
        'MembershipToComplete' => $MembershipToComplete,
        'MembershipFirstAmount' => $MembershipFirstAmount,
        'FrequenciesData' => $FrequenciesData,
        'currency' => [
            'code' => isset($vars['currency']) ? $vars['currency'] : 'EUR',
            'symbol' => isset($vars['currencySymbol']) ? $vars['currencySymbol'] : '€',
        ],
        'i18n' => [
            'dateformat' => E::ts('dd MM'),
            'my_contribution' => E::ts('My Contribution'),
            'your_salary_tax_free' => E::ts('Your monthly salary, tax free'),
            'pick_your_salary' => E::ts('Pick your salary'),
            'greater_salary' => E::ts('Greater'),
            'another_salary' => E::ts('Fill another salary amount'),
            'custom_salary' => E::ts('Please fill your monthly salary'),
            'custom_salary_nan' => E::ts('Please fill a number'),
            'your_contribution_mode' => E::ts('Your contribution mode'),
            'membership_completion' => E::ts('An error occured on payments on this membership. You can complete your membership by paying the remaining amount.'),
            'first_transaction' => E::ts('First transaction amount:'),
            'number_of_transactions' => E::ts('(corresponding to %d transactions)'),
            'next_transactions' => E::ts('Then, every %d %s:'),
            'amount_per_trans' => E::ts('Amount per transaction:'),
            'year_s' => E::ts('year(s)'),
            'month_s' => E::ts('month(s)'),
            'week_s' => E::ts('week(s)'),
            'day_s' => E::ts('day(s)'),
        ],
    ];
    if(civicrm_api3('Setting', 'getvalue', [
        'return' => "value",
        'name' => "debug_enabled",
    ])){
        $variables['debug'] = [
            '_PriceSet' => isset($vars['priceSet']) ? $vars['priceSet'] : null,
            '_RatesSettings' => $membershipRatesSettings,
        ];
    }
    \WPReporting()->stop();

    return $variables;
}

function cartable_get_indexed_syndicates_and_provinces($vars){
    \WPReporting()->listen('cartable', E_ALL);

    $PaymentProcessorsField = cartable_get_custom_field_id('PaymentProcessors', 'DonneesSyndicat');
    $ProvinceField = cartable_get_custom_field_id('Departements', 'DonneesSyndicat');
    $RulesOfProcedureField = cartable_get_custom_field_id('RulesOfProcedure', 'DonneesSyndicat');
    $AcceptRulesOfProcedureField = cartable_get_custom_field_id('AcceptRulesOfProcedure', 'DonneesParticulier');
    $FrequencesPaiementField = cartable_get_custom_field_id('FrequencesPaiement', 'ParametresAdhesion');
    $GrilleCotisationField = cartable_get_custom_field_id('GrilleCotisation', 'ParametresAdhesion');
    $RatioSupplementaireField = cartable_get_custom_field_id('RatioSupplementaire', 'ParametresAdhesion');
    $ContributionStartDayField = cartable_get_custom_field_id('ContributionStartDay', 'ParametresAdhesion');
    $ContributionStartMonthField = cartable_get_custom_field_id('ContributionStartMonth', 'ParametresAdhesion');
    $ContributionEndDayField = cartable_get_custom_field_id('ContributionEndDay', 'ParametresAdhesion');
    $ContributionEndMonthField = cartable_get_custom_field_id('ContributionEndMonth', 'ParametresAdhesion');
    $ProRataField = cartable_get_custom_field_id('ProRata', 'ParametresAdhesion');
    if(!$ProvinceField){
        return false;
    }
    // First get all needed values
    $CountryId = civicrm_api3('Country', 'getvalue', ['return'=>"id", 'name'=>"france"]);
    $Provinces = civicrm_api3('StateProvince', 'get', ['country_id'=>$CountryId, 'options'=>['limit'=>0], 'sequential'=>1]);
    $Syndicates = civicrm_api3('Contact', 'get', ['return'=>"id,display_name,{$ProvinceField},{$FrequencesPaiementField},{$GrilleCotisationField},{$RatioSupplementaireField},{$PaymentProcessorsField},{$RulesOfProcedureField},{$ContributionStartDayField},{$ContributionStartMonthField},{$ContributionEndDayField},{$ContributionEndMonthField},{$ProRataField}", 'contact_sub_type'=>"SyndicatSUD", 'options'=>['limit'=>-1], 'sequential'=>1 ]);
    $MembershipTypes = civicrm_api3('MembershipType', 'get', ['options'=>['limit'=>0]]);
    $defaultRulesOfProcedure = nl2br(civicrm_api3('Setting', 'getvalue', [
        'return' => "value",
        'name' => "cartable_rules_of_procedure",
    ]));

    $membershipFieldID = isset($vars['membershipFieldID']) ? $vars['membershipFieldID'] : null;

    $SyndicatesList = [];
    $IndexedSyndicates = [];
    $IndexedProvinces = [];
    $IndexedMembershipTypes = [];
    $IndexedMembershipFields = [];

    // Index values per relation
    foreach($Provinces['values'] as $province){
        $IndexedSyndicates[$province['abbreviation']] = [];
        $IndexedProvinces[$province['id']] = $province['abbreviation'];
    }
    foreach($MembershipTypes['values'] as $index=>$membershipType){
        unset($membershipType['is_active']);
        unset($membershipType['visibility']);
        unset($membershipType['domain_id']);
        unset($membershipType['contribution_type_id']);
        unset($membershipType['weight']);
        if(isset($membershipType['relationship_direction'])){
            unset($membershipType['relationship_direction']);
        }
        if(isset($membershipType['relationship_type_id'])){
            unset($membershipType['relationship_type_id']);
        }
        $next_renew_date = strtotime('+'.$membershipType['duration_interval'].' '.$membershipType['duration_unit']);
        if($membershipType['period_type'] == 'fixed'){
            if(!isset($membershipType['fixed_period_start_day'])){
                $membershipType['fixed_period_start_day'] = '101';
            }
            if(!isset($membershipType['fixed_period_rollover_day'])){
                $membershipType['fixed_period_rollover_day'] = '1231';
            }
            $this_year = date('Y');
            $next_year = date('Y', $next_renew_date);
            $membershipType['fixed_period_start_day'] = str_pad($membershipType['fixed_period_start_day'], 4, '0', STR_PAD_LEFT);
            $membershipType['fixed_period_rollover_day'] = str_pad($membershipType['fixed_period_rollover_day'], 4, '0', STR_PAD_LEFT);
            $membershipType['start_day'] = $next_year.'-'.substr($membershipType['fixed_period_start_day'], 0, 2).'-'.substr($membershipType['fixed_period_start_day'], 2, 2);
            $membershipType['end_day'] = $this_year.'-'.substr($membershipType['fixed_period_rollover_day'], 0, 2).'-'.substr($membershipType['fixed_period_rollover_day'], 2, 2);
            unset($membershipType['fixed_period_start_day']);
            unset($membershipType['fixed_period_rollover_day']);
        }
        elseif($membershipType['period_type'] == 'rolling'){
            $membershipType['start_day'] = date('Y-m-d', $next_renew_date);
            $membershipType['end_day'] = date('Y-m-d', strtotime(' -1 day'));
        }
        if(isset($membershipType['member_of_contact_id']) && $membershipType['member_of_contact_id']){
            $IndexedMembershipTypes[$membershipType['member_of_contact_id']] = $membershipType;
        }
        $MembershipTypes['values'][$index] = $membershipType;
    }
    if(isset($vars['priceSet']['fields'][$membershipFieldID]['options'])){
        foreach($vars['priceSet']['fields'][$membershipFieldID]['options'] as $membershipField){
            if(isset($membershipField['membership_type_id'])){
                $IndexedMembershipFields[$membershipField['membership_type_id']] = $membershipField;
            }
        }
    }

    // Aggregate syndicates per department, membership types per syndicate
    foreach($Syndicates['values'] as $syndicate){
        if(!isset($syndicate[$ProvinceField]) || empty($syndicate[$ProvinceField])){
            continue;
        }
        //CIVICRM_QFID_15_price_12
        //CIVICRM_QFID_{$membershipFieldOpeionId}_price_{$membershipFieldID}
        $syndicate['membership_field'] = ( isset($IndexedMembershipTypes[$syndicate['id']]) && isset($IndexedMembershipFields[$IndexedMembershipTypes[$syndicate['id']]['id']]))
            ? 'CIVICRM_QFID_'.$IndexedMembershipFields[$IndexedMembershipTypes[$syndicate['id']]['id']]['id'].'_price_'.$IndexedMembershipFields[$IndexedMembershipTypes[$syndicate['id']]['id']]['price_field_id']
            : null;
        $syndicate['id'] = (int) $syndicate['id'];
        $syndicate['contact_id'] = (int) $syndicate['contact_id'];
        $syndicate['PaymentProcessors'] = $syndicate[$PaymentProcessorsField] ? array_map(function($p){return (int) $p;},explode(',', $syndicate[$PaymentProcessorsField])) : [];
        $syndicate['Departements'] = array_map(function($p){return (int) $p;},$syndicate[$ProvinceField]);
        $syndicate['PaymentFrequency'] = $syndicate[$FrequencesPaiementField];
        $syndicate['MembershipRates'] = cartable_parse_rates($syndicate[$GrilleCotisationField]);
        $syndicate['membershipRatio'] = (float) $syndicate[$RatioSupplementaireField];
        $syndicate['RulesOfProcedure'] = $syndicate[$RulesOfProcedureField];
        $syndicate['ContributionStartDay'] = $syndicate[$ContributionStartDayField] ? str_pad($syndicate[$ContributionStartDayField], 2, 0, STR_PAD_LEFT) : null;
        $syndicate['ContributionStartMonth'] = $syndicate[$ContributionStartMonthField] ? str_pad($syndicate[$ContributionStartMonthField], 2, 0, STR_PAD_LEFT) : null;
        $syndicate['ContributionEndDay'] = $syndicate[$ContributionEndDayField] ? str_pad($syndicate[$ContributionEndDayField], 2, 0, STR_PAD_LEFT) : null;
        $syndicate['ContributionEndMonth'] = $syndicate[$ContributionEndMonthField] ? str_pad($syndicate[$ContributionEndMonthField], 2, 0, STR_PAD_LEFT) : null;
        $syndicate['ProRata'] = (bool) $syndicate[$ProRataField];
        unset($syndicate[$PaymentProcessorsField]);
        unset($syndicate[$ProvinceField]);
        unset($syndicate[$FrequencesPaiementField]);
        unset($syndicate[$GrilleCotisationField]);
        unset($syndicate[$RatioSupplementaireField]);
        unset($syndicate[$RulesOfProcedureField]);
        unset($syndicate[$ContributionStartDayField]);
        unset($syndicate[$ContributionStartMonthField]);
        unset($syndicate[$ContributionEndDayField]);
        unset($syndicate[$ContributionEndMonthField]);
        unset($syndicate[$ProRataField]);
        unset($syndicate['contact_sub_type']);
        unset($syndicate['contact_is_deleted']);

        $syndicate_provinces = $syndicate['Departements'];

        $SyndicatesList[$syndicate['id']] = $syndicate;
        foreach($syndicate['Departements'] as $province_id){
            $IndexedSyndicates[$IndexedProvinces[$province_id]][] = $syndicate['id'];
        }
    }

    $variables = [
        'Syndicates' => $SyndicatesList,
        'defaultRulesOfProcedure' => $defaultRulesOfProcedure,
        'AcceptRulesOfProcedureField' => $AcceptRulesOfProcedureField,
        'IndexedSyndicates' => $IndexedSyndicates,
        'IndexedProvinces' => $IndexedProvinces,
        'IndexedMembershipTypes' => $IndexedMembershipTypes,
        'i18n' => [
            'tabs' => [
                'back' => E::ts('« Back'),
                'next' => E::ts('Next »'),
            ],
            'seeRulesOfProcedure' => E::ts('See the rules of procedure'),
        ],
    ];

    if(civicrm_api3('Setting', 'getvalue', [
        'return' => "value",
        'name' => "debug_enabled",
    ])){
        $variables['debug'] = [
            '_Provinces' => $Provinces['values'],
            '_Syndicates' => $Syndicates['values'],
            '_MembershipTypes' => $MembershipTypes['values'],
            '_PriceSet' => isset($vars['priceSet']) ? $vars['priceSet'] : null,
            '_MembershipFields' => (isset($vars['priceSet']) && isset($vars['priceSet']['fields'][$membershipFieldID])) ? $vars['priceSet']['fields'][$membershipFieldID] : null,
            '_Vars' => $vars,
        ];
    }
    \WPReporting()->stop();

    return $variables;
}


/**
 * Get Wordpress user id from civiId
 * @param int $contactID
 * @return mixed
 */
function getWordPressUserID(int $contactID) {
    global $wpdb;
    $tableName = 'civicrm_uf_match'; 
    
    $user_ufmatch_dao = CRM_Core_DAO::executeQuery("SELECT uf_id FROM {$tableName} WHERE contact_id={$contactID}");
    $user_ufmatch = $user_ufmatch_dao->fetchAll();
    $userID = isset($user_ufmatch[0]['uf_id']) ? $user_ufmatch[0]['uf_id'] : null;
    return $userID;
}

function cartable_getSvgCard() {
    $svgFilePath = cartable_get_file_path('assets/academies.svg');
    $svgContent = file_get_contents($svgFilePath);
    return $svgContent;
}

/**
 * Get actives departments
 * @return array
 */
function cartable_get_actives_departments() {
    $actives_syndicats = \Civi\Api4\Contact::get(false)
        ->addSelect('display_name', 'DonneesSyndicat.Departements')
        ->addWhere('contact_sub_type', '=', 'SyndicatSUD')
        ->addWhere('DonneesSyndicat.MembershipPage', 'IS NOT NULL')
        ->addWhere('DonneesSyndicat.MembershipPage', '!=', '')
        ->execute();

    $actives_departments = [];

    foreach ($actives_syndicats as $syndicat) {
        if (is_array($syndicat) && isset($syndicat['DonneesSyndicat.Departements'])) {
            $departements = $syndicat['DonneesSyndicat.Departements'];

            foreach ($departements as $departementId) {
                $stateProvinces = \Civi\Api4\StateProvince::get(false)
                    ->addSelect('abbreviation')
                    ->addWhere('id', '=', $departementId)
                    ->execute();

                if (!empty($stateProvinces[0]['abbreviation'])) {
                    $actives_departments[] = $stateProvinces[0]['abbreviation'];
                }
            }
        }
    }
    return $actives_departments;
}

/**
 * Get department of connected user
 * @return mixed
 */
function cartable_get_department_connected_contact(){
    $loggedInContactId = CRM_Core_Session::getLoggedInContactID();
    if($loggedInContactId) {
        $department = \Civi\Api4\Contact::get(false)
            ->addSelect('address_primary.state_province_id:abbr')
            ->addWhere('contact_type', '=', 'Individual')
            ->addWhere('id', '=', $loggedInContactId)
            ->execute();
        $user_dep = $department[0]["address_primary.state_province_id:abbr"];
        return $user_dep;
    }
    return false;
}

/**
 * Get syndicats and department abbreviation
 * @return array
 */
function cartable_map_syndicats(){
    $syndicats = \Civi\Api4\Contact::get(false)
        ->addSelect('DonneesSyndicat.Departements', 'display_name')
        ->addWhere('contact_sub_type', '=', 'SyndicatSUD')
        ->execute();

    $associativeArray = [];

    foreach ($syndicats as $syndicat) {
        $displayName = $syndicat['display_name'];
        $departments = $syndicat['DonneesSyndicat.Departements'];
        if(is_array($departments)) {
            foreach ($departments as $dep) {
                $stateProvinces = \Civi\Api4\StateProvince::get(false)
                ->addSelect('abbreviation', 'id')
                ->addWhere('id', '=', $dep)
                ->execute();
                if (!empty($stateProvinces)) {
                    $abbreviation = $stateProvinces[0]['abbreviation'];
                    $id = $stateProvinces[0]['id'];

                    if (!isset($associativeArray[$abbreviation])) {
                        $associativeArray[$abbreviation] = [];
                    }

                    $associativeArray[$abbreviation][] = [
                        'id' => $id,
                        'display_name' => $displayName
                    ];
                }
            }
        }
    }
    return $associativeArray;
}

/**
 * Get MembershipPages and DonationPages per department id
 * @return array<array>
 */
function cartable_get_syndicats_pages() {
    $syndicats_data = \Civi\Api4\Contact::get(false)
        ->addSelect('id','DonneesSyndicat.MembershipPage', 'DonneesSyndicat.DonationPage', 'DonneesSyndicat.Departements', 'DonneesSyndicat.External_URL')
        ->addWhere('contact_sub_type', '=', 'SyndicatSUD')
        ->execute();

    $syndicats_pages = [];

    foreach ($syndicats_data as $syndicat) {
        $externalURL = $syndicat['DonneesSyndicat.External_URL'];
        $departements = $syndicat['DonneesSyndicat.Departements'];
        $membershipPage = $syndicat['DonneesSyndicat.MembershipPage'];
        $donationPage = $syndicat['DonneesSyndicat.DonationPage'];
        $syndicatId = $syndicat['id'];

        if (is_array($departements)) {
            foreach ($departements as $department) {
                if ($externalURL === null || $externalURL === "") {
                    $syndicats_pages[$department] = [
                        'MembershipPage' => $membershipPage,
                        'DonationPage' => $donationPage,
                        'ExternalURL' => null,
                        'syndicat_id' => $syndicatId
                    ];
                } else {
                    $syndicats_pages[$department] = [
                        'ExternalURL' => $externalURL,
                        'syndicat_id' => $syndicatId
                    ];
                }
            }
        }
    }

    return $syndicats_pages;
}

/**
 * Get the membership to complete id from the url or the post
 * @return int|null
 */
function cartable_get_membership_to_complete_id(){
    $MembershipToComplete_id = null;
    if(isset($_GET['membership-to-complete'])){
        $MembershipToComplete_id = filter_input(INPUT_GET, 'membership-to-complete', FILTER_SANITIZE_NUMBER_INT);
    }
    if(isset($_POST['membership-to-complete'])){
        $MembershipToComplete_id = filter_input(INPUT_POST, 'membership-to-complete', FILTER_SANITIZE_NUMBER_INT);
    }
    return $MembershipToComplete_id;
}

/**
 * Get the membership to complete id from the url or the post
 * @return float|null
 */
function cartable_get_membership_first_amount(){
    $MembershipFirstAmount = null;
    if(isset($_GET['membership-first-amount'])){
        $MembershipFirstAmount = $_GET['membership-first-amount'];
    }
    if(isset($_POST['membership-first-amount'])){
        $MembershipFirstAmount = $_POST['membership-first-amount'];
    }
    if($MembershipFirstAmount){
        $MembershipFirstAmount = filter_var($MembershipFirstAmount, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
    }
    return $MembershipFirstAmount;
}

/**
 * Get data of the membership to complete
 * @param int $MembershipToComplete_id
 * @return array|null
 */
function cartable_get_membership_to_complete(int $MembershipToComplete_id){
    $MembershipToComplete = null;
    // Get membership with api4
    $RawMembershipToComplete = \Civi\Api4\Membership::get(false)
        // Basic fields
        ->addSelect('id')
        ->addSelect('contact_id')
        ->addSelect('status_id:name')
        // Custom fields
        ->addSelect('membership_type_id:name')
        ->addSelect('Parametres.GrilleSalariale')
        ->addSelect('Parametres.numberOfMonthsForProrata')
        ->addSelect('Parametres.PaymentMethod')
        ->addSelect('Parametres.numberOfMonthsForContribution')
        // Contribution fields
        ->addSelect('contribution_recur_id')
        ->addSelect('contribution_recur_id.amount')
        ->addSelect('contribution_recur_id.frequency_unit')
        ->addSelect('contribution_recur_id.frequency_interval')
        ->addSelect('contribution_recur_id.payment_processor_id')
        ->addSelect('contribution_recur_id.start_date')

        ->addWhere('id', '=', $MembershipToComplete_id)
        ->execute()
        ->first();

    if($RawMembershipToComplete){
        // If membership is owned by another contact, bypass
        if($RawMembershipToComplete['contact_id'] == CRM_Core_Session::singleton()->getLoggedInContactID()){
            $MembershipToComplete = [
                'id' => $RawMembershipToComplete['id'],
                'contact_id' => $RawMembershipToComplete['contact_id'],
                'status' => $RawMembershipToComplete['status_id:name'],

                'GrilleSalariale' => $RawMembershipToComplete['Parametres.GrilleSalariale'],
                'PaymentMethod' => $RawMembershipToComplete['Parametres.PaymentMethod'],
                'numberOfMonthsForProrata' => $RawMembershipToComplete['Parametres.numberOfMonthsForProrata'] ? (int) $RawMembershipToComplete['Parametres.numberOfMonthsForProrata'] : null,
                'numberOfMonthsForContribution' => $RawMembershipToComplete['Parametres.numberOfMonthsForContribution'] ? (int) $RawMembershipToComplete['Parametres.numberOfMonthsForContribution'] : null,

                'contribution_recur_id' => $RawMembershipToComplete['contribution_recur_id'],
                'recur_amount' => $RawMembershipToComplete['contribution_recur_id.amount'],
                'frequency_unit' => $RawMembershipToComplete['contribution_recur_id.frequency_unit'],
                'frequency_interval' => $RawMembershipToComplete['contribution_recur_id.frequency_interval'],
                'payment_processor_id' => $RawMembershipToComplete['contribution_recur_id.payment_processor_id'],

                'first_transaction' => $RawMembershipToComplete['contribution_recur_id.amount'],
            ];
            if(strstr($RawMembershipToComplete['Parametres.GrilleSalariale'], '|')){
                $MembershipToCompleteParametresGrilleSalariale = explode('|', $RawMembershipToComplete['Parametres.GrilleSalariale']);
                $MembershipToComplete['expected_total_amount'] = (float) $MembershipToCompleteParametresGrilleSalariale[1];
            }
            $recur_start_date = strtotime($RawMembershipToComplete['contribution_recur_id.start_date']);
            $day_of_month = date('d', $recur_start_date);
            $current_day = date('d');
            if($day_of_month >= $current_day){
                $now = strtotime('last day of this month midnight');
            }
            else{
                $now = time();
            }
            $nbr_of_due_installments = 0;
            $diff = $now - $recur_start_date; 
            // Check due installments depending on frequency and start date
            if($RawMembershipToComplete['contribution_recur_id.frequency_unit'] == 'day'){
                $nbr_of_due_installments = floor($diff / (1 * 24 * 60 * 60));
            }
            if($RawMembershipToComplete['contribution_recur_id.frequency_unit'] == 'week'){
                $nbr_of_due_installments = floor($diff / (7 * 24 * 60 * 60));
            }
            if($RawMembershipToComplete['contribution_recur_id.frequency_unit'] == 'month'){
                $nbr_of_due_installments = floor($diff / (30 * 24 * 60 * 60));
            }
            else if($RawMembershipToComplete['contribution_recur_id.frequency_unit'] == 'year'){
                $nbr_of_due_installments = floor($diff / (365 * 24 * 60 * 60));
            }
            if($nbr_of_due_installments > 0){
                $MembershipToComplete['first_transaction'] = round($RawMembershipToComplete['contribution_recur_id.amount'] * $nbr_of_due_installments, 2);
                $MembershipToComplete['nbr_due_installments'] = $nbr_of_due_installments;
            }
        }
    }
    return $MembershipToComplete;
}

/**
 * Log debug in a file
 * @param string $message
 * @param string|array|object $data
 * 
 */
function cartable_log(string $message, $data = null){
    $log = "[Cartable]\n{$message}\n";
    if($data){
        $log .= print_r($data, true)."\n";
    }
    Civi::log()->debug($log);
}

/**
 * 
 */
function _cartable_get_option_group_id($option_group_name, $custom_group_name){
    $Field = \Civi\Api4\CustomField::get()
    ->addSelect('option_group_id')
    ->addWhere('name', '=', $option_group_name)
    ->addWhere('custom_group_id:name', '=', $custom_group_name)
    ->setLimit(1)
    ->execute();
    return isset($Field[0]['option_group_id']) ? $Field[0]['option_group_id'] : null;
}

/**
 * Cleanup new values in order to prevent "Already exists" error
 * @param array $options
 * @param int $option_group_id
 * @return array
 */
function _cartable_cleanup_existing_values_in_options(array $options, int $option_group_id){
  if(!$option_group_id){
    error_log('Cartable: Option group id is not set');
    return [];
  }
  $optionValues = \Civi\Api4\OptionValue::get()
    ->addSelect('value', 'label')
    ->addWhere('option_group_id', '=', $option_group_id)
    ->execute();
  foreach ($optionValues as $optionValue) {
    if (array_key_exists($optionValue['value'], $options)) {
      unset($options[$optionValue['value']]);
    }
  }
  return $options;
}

/**
 * Filters options by returning only ones existing in given optionGroup
 * @param array $options
 * @param int $option_group_id
 * @return array
 */
function _cartable_remaining_values_in_options(array $options_to_delete, int $option_group_id){
  if(!$option_group_id){
    error_log('Cartable: Option group id is not set');
    return [];
  }
  $optionValues = \Civi\Api4\OptionValue::get()
    ->addSelect('value', 'label')
    ->addWhere('option_group_id', '=', $option_group_id)
    ->execute();
  $options = [];
  foreach ($optionValues as $optionValue) {
    if (array_key_exists($optionValue['value'], $options_to_delete)) {
      $options[$optionValue['value']] = $optionValue['label'];
    }
  }
  return $options;
}


/**
 * Generates a values for `vads_sub_desc` parameter
 * Based on iCalendar standard RFC5545
 * Manages "end of month" case for inexisting days, aka february 30th.
 *
 * @see http://tools.ietf.org/html/rfc5545
 * @param string $effet_date yyyy-mm-dd format
 * @param string $frequency (DAILY, MONTHLY, YEARLY)
 * @param int $interval
 *
 * @return string
 */
function cartable_generateRrule(string $effet_date, string $frequency, int $interval): string
{
    $rrule = "RRULE:FREQ=$frequency;INTERVAL=$interval;";
    $month = substr($effet_date, 5, 2);
    $day_of_month = substr($effet_date, 8, 2);
    if($frequency=='YEARLY') {
        $rrule .= 'BYMONTH='.abs($month).';';
        if($month == '02' && $day_of_month == 29){
            $rrule .= 'BYSETPOS=-1;BYMONTHDAY=28,29;';
        }
        else{
            $rrule .= 'BYMONTHDAY='.abs($day_of_month).';';
        }
    }
    elseif($day_of_month == 31) {
        $rrule .= 'BYSETPOS=-1;BYMONTHDAY=28,29,30,31;';
    }
    elseif($day_of_month == 30) {
        $rrule .= 'BYSETPOS=-1;BYMONTHDAY=28,29,30;';
    }
    elseif($day_of_month == 29) {
        $rrule .= 'BYSETPOS=-1;BYMONTHDAY=28,29;';
    }
    return $rrule;
}

/**
 * Get the next payment date for a given date and a given break-range
 * 
 * @param string $return_date The current contribution date
 * @param int $month_start The start month of the contribution range
 * @param int $day_start The start day of the contribution range
 * @param int $month_end The end month of the contribution range
 * @param int $day_end The end day of the contribution range
 * 
 * @return string
 */
function cartable_compute_next_contribution_date(string $return_date, int $month_start, int $day_start, int $month_end, int $day_end): string{
    $date = strtotime($return_date);
    $monthday_current = (int) date('md', $date);

    $monthday_period_start = (int) ($month_start.str_pad($day_start, 2, '0', STR_PAD_LEFT));
    $monthday_period_end   = (int) ($month_end.str_pad($day_end, 2, '0', STR_PAD_LEFT));

    // Between january 1rst & range end
    if($monthday_current >= 101 && $monthday_current <= $monthday_period_end) {
        // $next_start_date = mktime(0, 0, 0, $month_start, $day_start, date('Y', $date));
        // Do not alter the date
    }
    // Out of range
    elseif($monthday_current > $monthday_period_end && $monthday_current < $monthday_period_start) {
        $next_start_date = mktime(0, 0, 0, $month_start, $day_start, date('Y', $date));
        $return_date = date('Y-m-d', $next_start_date);
    }
    // Between range start & december 31rst
    elseif($monthday_current > $monthday_period_end) {
        // $next_start_date = mktime(0, 0, 0, $month_start, $day_start, date('Y', $date)+1);
        // Do not alter the date
    }

    return $return_date;
}