document.addEventListener('DOMContentLoaded', function () {
    //loading
    const loadingElement = document.createElement('div');
    loadingElement.textContent = ts('Loading...');
    loadingElement.style.position = 'fixed';
    loadingElement.style.top = '50%'; 
    loadingElement.style.left = '50%';
    loadingElement.style.transform = 'translate(-50%, -50%)';
    loadingElement.style.backgroundColor = 'rgba(255, 255, 255, 0.8)';
    loadingElement.style.border = '1px solid #ccc'; 
    loadingElement.style.padding = '10px 20px';
    loadingElement.style.borderRadius = '5px';
    loadingElement.style.boxShadow = '0 0 10px rgba(0, 0, 0, 0.2)';
    loadingElement.style.display = 'none';
    document.body.appendChild(loadingElement);

    //phone
    var inputs = CRM.$('input[id^="phone"]').not('[id*="ext"]');
    var regex = /^\+\d+$/;
    Array.from(inputs).forEach(function (input) {
        input.placeholder = '+33123456789';
        function validateInput() { 
            if (input.value.length > 0) {
                var originalValue = input.value;
                var modifiedValue = originalValue.replace(/[\s.-]/g, '');
                input.value = modifiedValue;
                if (!regex.test(input.value)) {
                    var translatedTextInvalidPhone = cartable.form_constrained_fields.l10n.invalidphone;
                    input.setCustomValidity(translatedTextInvalidPhone);
                    input.classList.add('crm-error');
                    CRM.alert(translatedTextInvalidPhone, 'Erreur', 'error');
                } else {
                    input.setCustomValidity('');
                    input.classList.remove('crm-error');
                }
            } else {
                input.setCustomValidity('');
                input.classList.remove('crm-error');
            }
        };
        input.addEventListener('blur', validateInput);
        input.form.addEventListener('submit', function (event) {
            validateInput();
            if (!input.checkValidity()) {
                event.preventDefault();
            }
        });
    });

    //city
    const cityInputs = CRM.$("input[id*=city]");
    cityInputs.autocomplete({
        source: function (request, response) {
            let inputValue = request.term;
            inputValue = inputValue.toUpperCase();
            if (inputValue.includes("ST ")) {
                inputValue = inputValue.replace("ST ", "SAINT-");
            }
            const apiUrl = `https://geo.api.gouv.fr/communes?nom=${encodeURIComponent(inputValue)}`;
            loadingElement.style.display = 'block';
    
            CRM.$.ajax({
                url: apiUrl,
                dataType: "json",
                success: function (data) {
                    const suggestions = data.map((feature) => {
                        return {
                            label: feature.nom,
                            value: feature.nom
                        };
                    });
    
                    response(suggestions);
                },
                error: function (error) {
                    console.error("Error fetching data:", error);
                },
                complete: function () {
                    loadingElement.style.display = 'none';
                }
            });
        }
    });
    cityInputs.on('blur', function () {
        let inputValue = CRM.$(this).val();
        //UPPER & NO ACCENT
        inputValue = removeAccents(inputValue.toUpperCase());
        CRM.$(this).val(inputValue);
    });

    function removeAccents(str) {
        return str.normalize("NFD").replace(/[\u0300-\u036f]/g, "");
    }
});