<?php


namespace Tests\Api;

use Tests\Support\ApiTester;

global $infos;

class PermissionsCest 
{
    public function _before(ApiTester $I)
    {
    }
    
    // tests
    public function tryToTest(ApiTester $I)
    {
        global $infos;
        $apikey = $I->get_apikey('admin');
        
        if($apikey) {
            // Authenticate
            $I->haveHttpHeader('Content-Type', 'application/x-www-form-urlencoded');
            $I->haveHttpHeader('X-Civi-Auth', "Bearer {$apikey}");

            // Get relationshipe types
            $result = $I->API4('RelationshipType', 'get', [
                'select' => [
                    'id',
                    'name_a_b',
                ],
                'where' => [
                    ['name_a_b', 'LIKE', 'SUD_%'],
                ],
            ]);
            $relationship_types = [];
            foreach($result['values'] as $relationship_type) {
                $relationship_types[$relationship_type['name_a_b']] = $relationship_type['id'];
            }
            codecept_debug($relationship_types);
            
            // Get managers
            $result = $I->API4('Relationship', 'get', [
                'select' => [
                    'id',
                    'contact_id_a',
                    'contact_id_a.display_name',
                    'contact_id_b',
                    'contact_id_b.display_name',
                    'contact_id_a.api_key',
                ],
                'where' => [
                    ['relationship_type_id', '=', $relationship_types['SUD_Gestionnaire']],
                    ['is_active', '=', true],
                    ['OR', [['end_date', 'IS NULL'], ['end_date', '>=', date('Y-m-d')]]],
                    ['NOT', [['contact_id_a.groups', 'IN', [1]]]],
                    ['contact_id_a.api_key', 'IS NOT NULL'],
                ],
                'groupBy' => [
                    'contact_id_b',
                ],
            ]);
            $syndicate_managers = [];
            foreach($result['values'] as $manager) {
                $syndicate_managers[] = [
                    'syndicate_id' => $manager['contact_id_b'],
                    'syndicate' => $manager['contact_id_b.display_name'],
                    'name' => $manager['contact_id_a.display_name'],
                    'apikey' => $manager['contact_id_a.api_key'],
                ];
            }
            codecept_debug($syndicate_managers);
            
            // Make tests for each manager
            foreach($syndicate_managers as $syndicate_id => $info) {
                // Count all contacts of type Individual having relationship with a syndicate
                $result = $I->API4('Relationship', 'get', [
                    'select' => [
                        'contact_id_a',
                        'contact_id_a.sort_name',
                    ],
                    'where' => [
                        ['relationship_type_id', 'IN', [$relationship_types['SUD_Membre_De'], $relationship_types['SUD_Contact_De']]],
                        ['is_active', '=', true],
                        ['contact_id_b', '=', $info['syndicate_id']],
                        ['contact_id_a.is_deleted', '=', false],
                    ],
                    'groupBy' => [
                        'contact_id_a',
                    ],
                ]);
                // Append contacts data to info
                $info['nb_all_contacts'] = $result['count'];
                $info['all_contacts'] = array_map(function($r){ 
                    return (int) $r['contact_id_a'];
                }, $result['values']);

                $infos = $info;
                $Manager = $I->haveFriend($info['name']);
                $Manager->does(function(ApiTester $I) {
                    global $infos;
                    codecept_debug($infos['nb_all_contacts']);
                    $syndicate_name = $infos['syndicate'];
                    $manager_name = $infos['name'];
                    $manager_apikey = $infos['apikey'];

                    // Authenticate as manager
                    codecept_debug("Testing manager {$syndicate_name}");
                    $I->haveHttpHeader('X-Civi-Auth', "Bearer {$manager_apikey}");
    
                    // Count all contacts of type Individual
                    $result = $I->API4('Contact', 'get', [
                        'select' => [
                            'id',
                            'sort_name',
                        ],
                        'where' => [
                            ['contact_type', '=', 'Individual']
                        ],
                    ]);
                    $nb_all_contacts = $result['count'];
                    codecept_debug($nb_all_contacts);

                    try{
                        $rs = $I->assertEquals($infos['nb_all_contacts'], $nb_all_contacts, "Manager {$manager_name} can only see contacts of syndicate {$syndicate_name}");
                    }
                    catch(\Exception $e) {
                        codecept_debug($e->getMessage());
                        $view_contact_ids = array_map(function($r){ 
                            return (int) $r['id'];
                        }, $result['values']);
                        if($infos['nb_all_contacts'] < $nb_all_contacts){
                            $diff = array_diff($view_contact_ids, $infos['all_contacts']);
                            codecept_debug($diff);
                            throw new \Exception("Manager {$manager_name} of syndicate {$syndicate_name} has more contacts than expected: ".implode(', ', $diff));
                        }
                        else{
                            $diff = array_diff($infos['all_contacts'], $view_contact_ids);
                            codecept_debug($diff);
                            throw new \Exception("Manager {$manager_name} of syndicate {$syndicate_name} has less contacts than expected: ".implode(', ', $diff));
                        }
                    }
                });
                $Manager->leave();
            }
        }
    }
}
