<?php


namespace Tests\Functional;

use Tests\Support\FunctionalTester;
use Codeception\Util\Locator;

require_once '_AdhesionProcess.php';
class AdhesionParSEPACest extends _AdhesionProcess
{
     /**
     * Check if the membership and the contribution are created
     * @param FunctionalTester $I
     */
    public function _checkupBefore(FunctionalTester $I)
    {
        // Check if the membership is created
        $I->click('#tab_member a');
        $I->wait(10);
        $I->see('Sud éducation 93', '#memberships table');
        $I->see('Nouveau', '#memberships table');
        
        // Check if the contribution is created
        $I->see('Contributions');
        $I->click('#tab_contribute a');
        $I->wait(10);
        $I->see('Contribution en ligne: Adhérer à Sud éducation 93', '.ui-tabs-panel');
        $I->see('En instance', '.ui-tabs-panel');
    }

    // tests
    public function _selectPayment(FunctionalTester $I)
    {
        $persona = $this->_Persona();
        $I->forceClick('input[placeholder="Je paierai par chèque"] + label');
        $I->wait(20);
        $I->forceClick('.payment_processor_sepa-direct-debit-ng + label');
        $I->wait(20);
        $I->fillField('bank_account_number', 'FR8818782605350470742990995');
        $I->fillField('bank_identification_number', 'AGRIFRPP839');
        $I->fillField('bank_name', 'BankName');
        $I->forceClick('#billingcheckbox');
        $I->setVal('select[title=Département]', $persona['state_province']);
        $I->see('Seine-Saint-Denis');
    }

    public function _validatePayment(FunctionalTester $I)
    {
        $I->click('_qf_Confirm_next');
    }

    public function _beforeTest(FunctionalTester $I)
    {
        # changer la date d'échéance "créditeur" avant le mandat sepa
        $this->_login($I, 'admin');
        $I->amOnPage('/wp-admin/admin.php?page=CiviCRM&q=civicrm%2Fadmin%2Fsetting%2Fsepa');
        $I->executeJS("fetchCreditor(4, false); cj('a.add').hide();");
        $I->waitForText('Ajouter/Modifier', 60);
        $I->waitForText('créancier', 60);
        $I->fillField('custom_cycledays', date('d', strtotime('+1 day')));
        $I->fillField('custom_OOFF_notice', '1');
        $I->click('Enregistrer');
        $this->_logout($I);
    }

    public function _completeContribution(FunctionalTester $I)
    {
        $I->click('#tab_sepa a');
        $I->waitForText('Prêt', 60);
        $I->amOnPage('/wp-admin/admin.php?page=CiviCRM&q=civicrm%2Fsepa');
        $I->forceClick('.icon.refresh-icon.ui-icon-refresh');
        $I->click('//tr[contains(., "TXG-4-OOFF")]//a[contains(@class, "button_close")]');
        $I->see('Fermer');
        $I->see('groupe SEPA');
        $I->click('Le fichier a été soumis avec succès');
        $I->see('est maintenant fermé');
        $I->click('.button_export');
        $this->_goToPersona($I);
    }

    public function _checkupAfter(FunctionalTester $I)
    {
        // Check if the contribution is completed
        $I->waitForElementClickable('#tab_contribute a', 60);
        $I->click('#tab_contribute a');
        $I->wait(10);
        $I->see('En cours', '.crm-contact-contribute-contributions');

        // Check if the membership is completed
        $I->click('#tab_member a');
        $I->wait(10);
        $I->see('Nouveau', '.ui-tabs-panel');
    }

    /**
     * After the cleanup
     * @param FunctionalTester $I
     */
    public function _afterCleanup(FunctionalTester $I)
    {
        $persona = $this->_Persona();
        $I->amOnPage('/wp-admin/admin.php?page=CiviCRM&q=civicrm%2Fcontact%2Fsearch&reset=1');
        $I->fillField('sort_name', $persona['email']);
        $I->click('Rechercher');
        $I->wait(5);
        $I->forceClick('button[data-op="yes"]');
        $I->waitForText("Aucun résultat", 60);
    }

    public function ProcessusAdhesion(FunctionalTester $I)
    {
        if(isset($_SERVER['CODECEPTION_SKIP_SEPA']) && $_SERVER['CODECEPTION_SKIP_SEPA'] == 'true'){
            codecept_debug('Skip SEPA test');
            return true;
        }
        
        parent::ProcessusAdhesion($I);
    }
}
