<?php


namespace Tests\Functional;

use Tests\Support\FunctionalTester;

abstract class _AdhesionProcess
{
    /**
     * @var array $persona
     */
    var $persona;

    public function _Persona(){
        if(empty($this->persona)){
            date_default_timezone_set('Europe/paris');
            $date = date('Y-m-d-H-i-s');
            $this->persona = [
                'first_name' => 'Test',
                'last_name' => 'Tester',
                'phone' => '0123456789',
                'display_name' => 'Tester, Test',
                'email' => str_replace('@', "+{$date}@", $_SERVER['CODECEPTION_USER_MAIL_ADDRESS']),
                'street_address' => '1 rue du test',
                'postal_code' => '93000',
                'city' => 'Test sur Seine',
                'state_province' => '2578',
                'statut' => 'stagiaire',
                'communication_preferences' => [
                    'sms' => true,
                    'email' => true,
                ],
                'accept_tc' => true,
                'accept_entity_tc' => true,
            ];
        }
        return $this->persona;
    }

    public function _login(FunctionalTester $I, string $role): bool {
        $classname = get_class($this);
        $this->role = strtolower(str_replace(['Acceptance', 'Cest'], '', $classname));
        
        $login = $I->get_login($role);
        $password = $I->get_password($role);
        
        if($login && $password) {
            $I->amOnPage('/');
            $I->see('Connexion');
            $I->seeLink('Connexion');
            $I->click('Connexion');
    
            $I->see('Identifiant ou adresse e-mail');
            $I->see('Mot de passe');
            $I->see('Se souvenir de moi');
            $I->fillField('log', $login);
            $I->fillField('pwd', $password);
            $I->click('Se connecter');
            return true;
        }
        return false;
    }

    public function _logout(FunctionalTester $I)
    {
        $I->amOnPage('/wp-login.php?action=logout');
        $I->click('vous déconnecter');
    }

    public function _selectPayment(FunctionalTester $I)
    {
    }

    public function _selectFrequency(FunctionalTester $I)
    {
        $I->click('.cartable-membership-frequency-selector.frequency-oneoff label');
    }

    public function _validatePayment(FunctionalTester $I)
    {
        $I->click('Suivant');
    }

    public function _beforeTest(FunctionalTester $I)
    {
    }
    
    public function _afterMap(FunctionalTester $I)
    {
    }

    /**
     * Subscribe on front with a fake user
     * @param FunctionalTester $I
     */
    private function _front(FunctionalTester $I)
    {
        $persona = $this->_Persona();

        // Carte
        $I->amOnPage('/');
        $I->see('J’adhère');
        $I->click('#zone-idf');
        $I->see('Sud éducation 93');
        $I->click('Sud éducation 93');
        
        $this->_afterMap($I);

        // Formulaire "Ma cotisation"
        $I->waitForText('Sélectionnez votre salaire');
        $I->click('Sélectionnez votre salaire');
        $I->click('.select2-results li:nth-child(2)');
        $I->wait(1);
        // $I->selectOption('#cartable-membership-frequency-selector input', 'oneoff');
        $this->_selectFrequency($I);
        $I->forceClick('#cartable-tab-step-2 .front-tabs-link.btn.next');
        
        // Formulaire "Mes informations"
        $I->wait(2);
        
        $I->setVal('#first_name', $persona['first_name']);
        $I->setVal('#last_name', $persona['last_name']);
        $I->setVal('#email-Primary', $persona['email']);
        $I->setVal('#street_address-1', $persona['street_address']);
        $I->setVal('#postal_code-1', $persona['postal_code']);
        $I->setVal('#city-1', $persona['city']);
        $I->setVal('#state_province-1', $persona['state_province']);
        $I->setVal('select[data-crm-custom=DonneesParticulier\\\\:Statut]', $persona['statut']);
        $I->click('input[data-crm-custom=Communication_Preferences\\:sms] + label');
        $I->click('input[data-crm-custom=Communication_Preferences\\:email] + label');
        $I->click('input#custom_125_1');
        
        // Formulaire "Mon paiement"
        $I->forceClick('#cartable-tab-step-3 .front-tabs-link.btn.next');
        $this->_selectPayment($I);
        $I->wait(3);
        $I->forceClick('input#accept_tc + label');
        $I->forceClick('input#accept_entity_tc + label');
        $I->wait('5');
        // Validation
        $I->click('Vérifiez votre contribution');
        $I->wait(5);
        $I->see('Veuillez vérifier attentivement les informations ci-dessous.');
        $this->_validatePayment($I);

        // Confirmation
        $I->see('Merci pour votre adhésion');
    }

    /**
     * Check if the membership and the contribution are created
     * @param FunctionalTester $I
     */
    public function _checkupBefore(FunctionalTester $I)
    {
        // Check if the membership is created
        $I->click('#tab_member a');
        $I->waitForText('Sud éducation 93', 10, '#inactive-memberships table');
        $I->see('En instance', '#inactive-memberships table');
        
        // Check if the contribution is created
        $I->see('Contributions');
        $I->click('#tab_contribute a');
        $I->wait(10);
        $I->see('Contribution en ligne: Adhérer à Sud éducation 93', '.ui-tabs-panel');
        $I->see('En instance', '.ui-tabs-panel');
    }

    /**
     * Complete the contribution
     * @param FunctionalTester $I
     */
    public function _completeContribution(FunctionalTester $I)
    {
    }

    /**
     * Check if the membership and the contribution are completed
     * @param FunctionalTester $I
     */
    public function _checkupAfter(FunctionalTester $I)
    {
        // Check if the contribution is completed
        $I->waitForElementClickable('#tab_contribute a');
        $I->click('#tab_contribute a');
        $I->waitForText('Terminé', 10, '.crm-contact-contribute-contributions');

        // Check if the membership is completed
        $I->click('#tab_member a');
        $I->wait(10);
        $I->see('Nouveau', '.ui-tabs-panel');
    }

    /**
     * Before the cleanup
     * @param FunctionalTester $I
     */
    public function _beforeCleanup(FunctionalTester $I)
    {
    }

    /**
     * Cleanup
     * @param FunctionalTester $I
     */
    public function _cleanup(FunctionalTester $I)
    {
        $I->click('#tab_member a');
        $I->wait(10);
        $I->forceClick('#memberships tr td:last-child span.crm-hover-button');
        $I->wait(5);
        $I->forceClick('#memberships tr td:last-child span.crm-hover-button .panel li a:contains("Supprimer")', 'CRM');
        $I->wait(5);
        $I->forceClick('.ui-dialog-buttons .ui-dialog-buttonset button:contains("Supprimer")', 'CRM');

        $I->wait(10);

        $persona = $this->_Persona();
        $I->amOnPage('/wp-admin/network/users.php');
        $I->wait(5);
        $I->fillField('#all-user-search-input', $persona['email']);
        $I->click('Rechercher des comptes');
        $I->forceClick('#cb-select-all-1 + label');
        $I->setVal('#bulk-action-selector-bottom', 'delete');
        $I->click('.button.action');
        $I->wait(5);
        $I->see('Vous avez choisi de supprimer le compte de tous les réseaux et sites.');
        $I->click('Confirmer cette action');
        $I->wait(5);
        $I->see('Compte supprimé.');
        
        $this->_goToPersona($I);
        $I->click('Supprimer contact');
        $I->click('Supprimer contact(s)');
        $I->click('Supprimer définitivement');
        $I->click('Supprimer contact(s)');
    }

    /**
     * After the cleanup
     * @param FunctionalTester $I
     */
    public function _afterCleanup(FunctionalTester $I)
    {
        $persona = $this->_Persona();
        $I->wait(5);
        $I->see("{$persona['first_name']} {$persona['last_name']} a été supprimé définitivement.");
    }
    
    public function _goToPersona(FunctionalTester $I)
    {
        $persona = $this->_Persona();
        $I->amOnPage('/wp-admin/admin.php?page=CiviCRM&q=civicrm%2Fcontact%2Fsearch&reset=1');
        $I->fillField('sort_name', $persona['email']);
        $I->click('Rechercher');
        $I->wait(5);
        $I->forceClick('button[data-op="yes"]');

        // Parse search results
        $I->wait(2);
        $I->see($persona['email']);
        $I->see($persona['display_name']);
        $I->click($persona['display_name']);
    }

    public function _afterMail(FunctionalTester $I)
    {
    }

    public function _checkMailConfirmation(FunctionalTester $I)
    {
        $apikey = $I->get_apikey('admin');
    
        // Authenticate
        $I->haveHttpHeader('Content-Type', 'application/x-www-form-urlencoded');
        $I->haveHttpHeader('X-Civi-Auth', "Bearer {$apikey}");

        // Get relationshipe types
        $messageTemplates = $I->API4('MessageTemplate', 'get', [
            'select' => [
                'msg_subject',
            ],
            'where' => [
                ['msg_title', '=', 'Contributions - Reçu de contribution contact'],
            ],
            'limit' => 25,
            'checkPermissions' => TRUE,
            ]);
        $mailSubject =($messageTemplates['values'][0]['msg_subject']);

        $persona = $this->_Persona();
        $I->amOnUrl($_SERVER['CODECEPTION_WEBMAIL_URL']);
        $I->wait(2);
        $I->fillField('_user', $_SERVER['CODECEPTION_USER_MAIL_ADDRESS']);
        $I->fillField('_pass', $_SERVER['CODECEPTION_USER_MAIL_PASSWORD']);
        $I->click('button[type=submit]');
        $I->see('test-sud@enprojet.xyz');
        $I->see($mailSubject);
        $I->click($mailSubject);
        $I->wait(2);
        $I->executeJS('UI.headers_show(true)');
        $I->see($persona['first_name']);
        $I->forceClick('#rcmbtn120');
        $this->_afterMail($I);
        $I->forceClick('.logout');
    }

    /**
     * Test the adhesion process
     * @param FunctionalTester $I
     */
    public function ProcessusAdhesion(FunctionalTester $I)
    {
        $this->_beforeTest($I);
        /**
         * -----------------------------------------------------------------------------------------
         * Generate data
         * -----------------------------------------------------------------------------------------
         */
        
        $this->_front($I);

        /**
         * -----------------------------------------------------------------------------------------
         * Backoffice checkups
         * -----------------------------------------------------------------------------------------
         */

        // Login as admin
        $this->_login($I, 'admin');
        
        
        // Search created contact
        $this->_goToPersona($I);

        $this->_checkupBefore($I);
        
        $this->_completeContribution($I);
        
        $this->_checkupAfter($I);
        
        /**
         * -----------------------------------------------------------------------------------------
         * Cleanup server
         * -----------------------------------------------------------------------------------------
         */
        
        $this->_beforeCleanup($I);
        
        $this->_cleanup($I);
        
        $this->_afterCleanup($I);

        $this->_logout($I);
        /**
         * -----------------------------------------------------------------------------------------
         * Checkup / cleanup mail
         * -----------------------------------------------------------------------------------------
         */
        $this->_checkMailConfirmation($I);
    }
}
