<?php

declare(strict_types=1);

namespace Tests\Support;

/**
 * Inherited Methods
 * @method void wantTo($text)
 * @method void wantToTest($text)
 * @method void execute($callable)
 * @method void expectTo($prediction)
 * @method void expect($prediction)
 * @method void amGoingTo($argumentation)
 * @method void am($role)
 * @method void lookForwardTo($achieveValue)
 * @method void comment($description)
 * @method void pause($vars = [])
 *
 * @SuppressWarnings(PHPMD)
*/
class _base extends \Codeception\Actor
{
    /**
     * Define the login for a given role
     */
    public function get_login(string $role){
        $var_login = 'CODECEPTION_USER_LOGIN_'.strtoupper($role);
        if(isset($_SERVER[$var_login])) {
            return $_SERVER[$var_login];
        }
        return null;
    }

    /**
     * Get the password for a given role
     */
    public function get_password(string $role){
        $var_password = 'CODECEPTION_USER_PASSWORD_'.strtoupper($role);
        if(isset($_SERVER[$var_password])) {
            return $_SERVER[$var_password];
        }
        return null;
    }

    /**
     * Get the password for a given role
     */
    public function get_apikey(string $role){
        $var_apikey = 'CODECEPTION_USER_APIKEY_'.strtoupper($role);
        if(isset($_SERVER[$var_apikey])) {
            return $_SERVER[$var_apikey];
        }
        return null;
    }

    /**
     * Get a fake IBAN number
     */
    public function get_iban(){
        return 'FR8289653626621143153448016';
    }

    /**
     * Get a fake BIC number
     */
    public function get_bic(){
        return 'CRLYFRPP';
    }

    /**
     * Force click on an "not interractable" element
     */
    public function forceClick(string $selector, $lib='vanilla'){
        $selector = addslashes($selector);
        if($lib === 'vanilla'){
            $selectorfunction = 'document.querySelector';
        }
        elseif($lib === 'jQuery'){
            $selectorfunction = 'jQuery';
        }
        elseif($lib === 'CRM'){
            $selectorfunction = 'CRM.$';
        }
        else{
            throw new \Exception('Unknown JS lib');
        }
        $this->executeJS("{$selectorfunction}(\"{$selector}\").click();");
    }

    /**
     * Set a value to an element
     */
    public function setVal(string $selector, string $value){
        $this->executeJS("jQuery('{$selector}').val(\"{$value}\").trigger('change');");
    }

    public function API4(string $entity, string $method, array $params)
    {
        $this->sendPost('/civicrm/ajax/api4/'. $entity .'/'. $method, [
            "params" => json_encode($params)
        ]);
        $this->seeResponseCodeIsSuccessful();
        $this->seeResponseIsJson();
        return json_decode($this->grabResponse(), true);
    }
}
